@@ -1,6 +1,6 @@
 ;;; cus-test.el --- tests for custom types and load problems  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1998, 2000, 2002-2022 Free Software Foundation, Inc.
+;; Copyright (C) 1998, 2000, 2002-2024 Free Software Foundation, Inc.
 
 ;; Author: Markus Rost <rost@math.uni-bielefeld.de>
 ;; Created: 13 Sep 1998
@@ -131,7 +131,7 @@ Names should be as they appear in loaddefs.el.")
 ;; Don't create a file `abbrev-file-name'.
 (setq save-abbrevs nil)
 
-;; Avoid compile logs from adviced functions.
+;; Avoid compile logs from advised functions.
 (eval-after-load "bytecomp"
   '(setq ad-default-compilation-action 'never))
 
@@ -145,7 +145,8 @@ Names should be as they appear in loaddefs.el.")
 (require 'cus-load)
 
 (defvar cus-test-errors nil
-  "List of problematic variables found by `cus-test-apropos'.")
+  "List of problematic variables found by `cus-test-apropos'.
+Each element is (VARIABLE . PROBLEM); see `cus-test--format-error'.")
 
 (defvar cus-test-tested-variables nil
   "List of options tested by last call of `cus-test-apropos'.")
@@ -156,7 +157,7 @@ Names should be as they appear in loaddefs.el.")
   "Set by `cus-test-apropos' to a list of options with :get property.")
 
 (defvar cus-test-vars-with-changed-state nil
-  "Set by `cus-test-apropos' to a list of options with state 'changed.")
+  "Set by `cus-test-apropos' to a list of options with state \\='changed.")
 
 (defvar cus-test-deps-errors nil
   "List of require/load problems found by `cus-test-deps'.")
@@ -181,6 +182,15 @@ Set by `cus-test-noloads'.")
 ;; (defvar cus-test-vars-cus-loaded nil
 ;;   "A list of options loaded by `custom-load-symbol'.")
 
+(defun cus-test--format-error (err)
+  "Format an element of `cus-test-errors'."
+  (pcase err
+    (`(,var :type-error ,value ,type)
+     (format "variable: %s\n   value: %S\n    type: %S" var value type))
+    (`(,var :other-error ,e)
+     (format "variable: %s\n   error: %S" var e))
+    (_ (format "%S" err))))
+
 (defun cus-test-apropos (regexp)
   "Check the options matching REGEXP.
 The detected problematic options are stored in `cus-test-errors'."
@@ -200,8 +210,7 @@ The detected problematic options are stored in `cus-test-errors'."
 	 (let* ((type (custom-variable-type symbol))
 		(conv (widget-convert type))
 		(get (or (get symbol 'custom-get) 'default-value))
-		values
-		mismatch)
+		values)
 	   (when (default-boundp symbol)
 	     (push (funcall get symbol) values)
 	     (push (eval (car (get symbol 'standard-value)) t) values))
@@ -212,10 +221,10 @@ The detected problematic options are stored in `cus-test-errors'."
 
 	   ;; Check the values
 	   (mapc (lambda (value)
-		   ;; TODO for booleans, check for values that can be
-		   ;; evaluated and are not t or nil.  Usually a bug.
 		   (unless (widget-apply conv :match value)
-		     (setq mismatch 'mismatch)))
+                     (let ((err (list symbol :type-error value type)))
+                       (unless (member err cus-test-errors)
+                         (push err cus-test-errors)))))
 		 values)
 
 	   ;; Store symbols with a custom-get property.
@@ -231,13 +240,12 @@ The detected problematic options are stored in `cus-test-errors'."
 	     (and (consp c-value)
 		  (boundp symbol)
 		  (not (equal (eval (car c-value) t) (symbol-value symbol)))
-		  (add-to-list 'cus-test-vars-with-changed-state symbol)))
-
-	   (if mismatch
-	       (push symbol cus-test-errors)))
+		  (add-to-list 'cus-test-vars-with-changed-state symbol))))
 
        (error
-	(push symbol cus-test-errors)
+        (let ((err (list symbol :other-error alpha)))
+          (unless (member err cus-test-errors)
+	    (push err cus-test-errors)))
 	(message "Error for %s: %s" symbol alpha))))
    (cus-test-get-options regexp))
   (message "%s options tested"
@@ -272,7 +280,7 @@ currently defined groups."
 	(if group
 	    (memq symbol groups)
 	  (or
-	   ;; (user-variable-p symbol)
+           ;; (custom-variable-p symbol)
 	   (get symbol 'standard-value)
 	   ;; (get symbol 'saved-value)
 	   (get symbol 'custom-type)))
@@ -292,7 +300,7 @@ currently defined groups."
 	(insert "No errors found by cus-test.")
       (insert "The following variables seem to have problems:\n\n")
       (dolist (e cus-test-errors)
-	(insert (symbol-name e) "\n")))))
+	(insert (cus-test--format-error e) "\n")))))
 
 (defun cus-test-load-custom-loads ()
   "Call `custom-load-symbol' on all atoms."
@@ -399,7 +407,7 @@ Returns a list of variables with suspicious types."
         (message "No problems found")
         nil)
     (message "The following options might have problems:")
-    (cus-test-message cus-test-errors)
+    (cus-test-message (mapcar #'cus-test--format-error cus-test-errors))
     cus-test-errors))
 
 (defun cus-test-deps ()
@@ -416,7 +424,12 @@ in the Emacs source directory."
   (mapatoms
    ;; This code is mainly from `custom-load-symbol'.
    (lambda (symbol)
-     (let ((custom-load-recursion t))
+     (let ((custom-load-recursion t)
+           (load-path
+            (cons
+             (expand-file-name
+              "quail" (file-name-directory (locate-library leim-list-file-name)))
+             load-path)))
        (dolist (load (get symbol 'custom-loads))
 	 (cond
 	  ((symbolp load)
