@@ -20,8 +20,18 @@
 #ifndef FILEVERCMP_H
 #define FILEVERCMP_H
 
+/* This file uses _GL_ATTRIBUTE_PURE.  */
+#if !_GL_CONFIG_H_INCLUDED
+ #error "Please include config.h first."
+#endif
+
 #include <stddef.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 /* Compare strings A and B as file names containing version numbers,
    and return an integer that is negative, zero, or positive depending
    on whether A compares less than, equal to, or greater than B.
@@ -75,4 +85,9 @@ int filevercmp (char const *a, char const *b) _GL_ATTRIBUTE_PURE;
 int filenvercmp (char const *a, ptrdiff_t alen, char const *b, ptrdiff_t blen)
   _GL_ATTRIBUTE_PURE;
 
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* FILEVERCMP_H */
