@@ -1,6 +1,6 @@
-;;; semantic/db-find.el --- Searching through semantic databases.
+;;; semantic/db-find.el --- Searching through semantic databases.  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2000-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2024 Free Software Foundation, Inc.
 
 ;; Author: Eric M. Ludlam <zappo@gnu.org>
 ;; Keywords: tags
@@ -209,14 +209,14 @@ This class will cache data derived during various searches.")
   )
 
 (cl-defmethod semanticdb-synchronize ((idx semanticdb-find-search-index)
-				   new-tags)
+				      _new-tags)
   "Synchronize the search index IDX with some NEW-TAGS."
   ;; Reset our parts.
   (semantic-reset idx)
   ;; Notify dependants by clearing their indices.
   (semanticdb-notify-references
    (oref idx table)
-   (lambda (tab me)
+   (lambda (tab _me)
      (semantic-reset (semanticdb-get-table-index tab))))
   )
 
@@ -230,7 +230,7 @@ This class will cache data derived during various searches.")
 	;; Notify dependants by clearing their indices.
 	(semanticdb-notify-references
 	 (oref idx table)
-	 (lambda (tab me)
+	 (lambda (tab _me)
 	   (semantic-reset (semanticdb-get-table-index tab))))
 	)
     ;; Else, not an include, by just a type.
@@ -240,7 +240,7 @@ This class will cache data derived during various searches.")
 	;; Notify dependants by clearing their indices.
 	(semanticdb-notify-references
 	 (oref idx table)
-	 (lambda (tab me)
+	 (lambda (tab _me)
 	   (let ((tab-idx (semanticdb-get-table-index tab)))
 	     ;; Not a full reset?
 	     (when (oref tab-idx type-cache)
@@ -315,7 +315,7 @@ Default action as described in `semanticdb-find-translate-path'."
 
 ;;;###autoload
 (define-overloadable-function semanticdb-find-table-for-include (includetag &optional table)
-  "For a single INCLUDETAG found in TABLE, find a `semanticdb-table' object
+  "For a single INCLUDETAG found in TABLE, find a `semanticdb-table' object.
 INCLUDETAG is a semantic TAG of class `include'.
 TABLE is a semanticdb table that identifies where INCLUDETAG came from.
 TABLE is optional if INCLUDETAG has an overlay of :filename attribute."
@@ -426,17 +426,15 @@ Default action as described in `semanticdb-find-translate-path'."
       ;; searchable item, then instead do the regular thing without caching.
       (semanticdb-find-translate-path-includes--internal path))))
 
-(defvar semanticdb-find-lost-includes nil
+(defvar-local semanticdb-find-lost-includes nil
   "Include files that we cannot find associated with this buffer.")
-(make-variable-buffer-local 'semanticdb-find-lost-includes)
 
-(defvar semanticdb-find-scanned-include-tags nil
+(defvar-local semanticdb-find-scanned-include-tags nil
   "All include tags scanned, plus action taken on the tag.
 Each entry is an alist:
   (ACTION . TAG)
 where ACTION is one of `scanned', `duplicate', `lost'
 and TAG is a clone of the include tag that was found.")
-(make-variable-buffer-local 'semanticdb-find-scanned-include-tags)
 
 (defvar semanticdb-implied-include-tags nil
   "Include tags implied for all files of a given mode.
@@ -624,7 +622,7 @@ Included databases are filtered based on `semanticdb-find-default-throttle'."
     ;; INCLUDETAG should have some way to reference where it came
     ;; from!  If not, TABLE should provide the way.  Each time we
     ;; look up a tag, we may need to find it in some relative way
-    ;; and must set our current buffer eto the origin of includetag
+    ;; and must set our current buffer to the origin of includetag
     ;; or nothing may work.
     (setq originfiledir
 	  (cond ((semantic-tag-file-name includetag)
@@ -793,7 +791,8 @@ PREBUTTONTEXT is some text between prefix and the overlay button."
 	 (file (semantic-tag-file-name tag))
 	 (str1 (format "%S %s" mode name))
 	 (str2 (format " : %s" file))
-	 (tip nil))
+	 ;; (tip nil)
+	 )
     (insert prefix prebuttontext str1)
     (setq end (point))
     (insert str2)
@@ -809,7 +808,7 @@ PREBUTTONTEXT is some text between prefix and the overlay button."
     (put-text-property start end 'ddebug (cdr consdata))
     (put-text-property start end 'ddebug-indent(length prefix))
     (put-text-property start end 'ddebug-prefix prefix)
-    (put-text-property start end 'help-echo tip)
+    ;; (put-text-property start end 'help-echo tip)
     (put-text-property start end 'ddebug-function
 		       'data-debug-insert-tag-parts-from-point)
     (insert "\n")
@@ -915,7 +914,7 @@ but should be good enough for debugging assertions."
 	   (null (car (cdr (car resultp)))))))
 
 (defun semanticdb-find-result-prin1-to-string (result)
-  "Presuming RESULT satisfies `semanticdb-find-results-p', provide a short PRIN1 output."
+  "If RESULT satisfies `semanticdb-find-results-p', provide a short PRIN1 output."
   (if (< (length result) 2)
       (concat "#<FIND RESULT "
 	      (mapconcat (lambda (a)
@@ -929,6 +928,9 @@ but should be good enough for debugging assertions."
 	    (semanticdb-find-result-length result)
 	    (length result))))
 
+(cl-deftype semanticdb-find-result-with-nil ()
+  '(satisfies semanticdb-find-result-with-nil-p))
+
 (defun semanticdb-find-result-with-nil-p (resultp)
   "Non-nil if RESULTP is in the form of a semanticdb search result.
 The value nil is valid where a TABLE usually is, but only if the TAG
@@ -1011,7 +1013,7 @@ is still made current."
 	  (when norm
 	    ;; The normalized tags can now be found based on that
 	    ;; tags table.
-	    (condition-case foo
+	    (condition-case nil
 		(progn
 		  (semanticdb-set-buffer (car norm))
 		  ;; Now reset ans
@@ -1245,7 +1247,7 @@ See `semanticdb-find-translate-path' for details on PATH.
 The argument BRUTISH will be set so that searching includes all tables
 in the current project.
 FIND-FILE-MATCH indicates that any time a match is found, the file
-associated wit that tag should be loaded into a buffer."
+associated with that tag should be loaded into a buffer."
   (semanticdb-find-tags-collector
    (lambda (table tags)
      (semanticdb-deep-find-tags-by-name-method table name tags))
@@ -1257,7 +1259,7 @@ See `semanticdb-find-translate-path' for details on PATH.
 The argument BRUTISH will be set so that searching includes all tables
 in the current project.
 FIND-FILE-MATCH indicates that any time a match is found, the file
-associated wit that tag should be loaded into a buffer."
+associated with that tag should be loaded into a buffer."
   (semanticdb-find-tags-collector
    (lambda (table tags)
      (semanticdb-deep-find-tags-for-completion-method table prefix tags))
@@ -1278,7 +1280,7 @@ associated with that tag should be loaded into a buffer."
 ;;; Specialty Search Routines
 (defun semanticdb-find-tags-external-children-of-type
   (type &optional path find-file-match)
-  "Search for all tags defined outside of TYPE w/ TYPE as a parent.
+  "Search for all tags defined outside of TYPE with TYPE as a parent.
 See `semanticdb-find-translate-path' for details on PATH.
 FIND-FILE-MATCH indicates that any time a match is found, the file
 associated with that tag should be loaded into a buffer."
@@ -1308,19 +1310,25 @@ associated with that tag should be loaded into a buffer."
   "In TABLE, find all occurrences of tags with NAME.
 Optional argument TAGS is a list of tags to search.
 Returns a table of all matching tags."
-  (semantic-find-tags-by-name name (or tags (semanticdb-get-tags table))))
+  (semantic-find-tags-by-name name
+                              (or tags (and (slot-boundp table 'tags)
+                                            (semanticdb-get-tags table)))))
 
 (cl-defmethod semanticdb-find-tags-by-name-regexp-method ((table semanticdb-abstract-table) regexp &optional tags)
   "In TABLE, find all occurrences of tags matching REGEXP.
 Optional argument TAGS is a list of tags to search.
 Returns a table of all matching tags."
-  (semantic-find-tags-by-name-regexp regexp (or tags (semanticdb-get-tags table))))
+  (semantic-find-tags-by-name-regexp regexp
+                                     (or tags (and (slot-boundp table 'tags)
+                                                   (semanticdb-get-tags table)))))
 
 (cl-defmethod semanticdb-find-tags-for-completion-method ((table semanticdb-abstract-table) prefix &optional tags)
   "In TABLE, find all occurrences of tags matching PREFIX.
 Optional argument TAGS is a list of tags to search.
 Returns a table of all matching tags."
-  (semantic-find-tags-for-completion prefix (or tags (semanticdb-get-tags table))))
+  (semantic-find-tags-for-completion prefix
+                                     (or tags (and (slot-boundp table 'tags)
+                                                   (semanticdb-get-tags table)))))
 
 (cl-defmethod semanticdb-find-tags-by-class-method ((table semanticdb-abstract-table) class &optional tags)
   "In TABLE, find all occurrences of tags of CLASS.
@@ -1330,8 +1338,12 @@ Returns a table of all matching tags."
   ;; `semantic-find-tags-included', which by default will just call
   ;; `semantic-find-tags-by-class'.
   (if (eq class 'include)
-      (semantic-find-tags-included (or tags (semanticdb-get-tags table)))
-    (semantic-find-tags-by-class class (or tags (semanticdb-get-tags table)))))
+      (semantic-find-tags-included
+       (or tags (and (slot-boundp table 'tags)
+                     (semanticdb-get-tags table))))
+    (semantic-find-tags-by-class class
+                                 (or tags (and (slot-boundp table 'tags)
+                                               (semanticdb-get-tags table))))))
 
 (declare-function semantic-find-tags-external-children-of-type
 		  "semantic/find" (type &optional table))
@@ -1341,7 +1353,9 @@ Returns a table of all matching tags."
 Optional argument TAGS is a list of tags to search.
 Returns a table of all matching tags."
    (require 'semantic/find)
-   (semantic-find-tags-external-children-of-type parent (or tags (semanticdb-get-tags table))))
+   (semantic-find-tags-external-children-of-type
+    parent (or tags (and (slot-boundp table 'tags)
+                         (semanticdb-get-tags table)))))
 
 (declare-function semantic-find-tags-subclasses-of-type
 		  "semantic/find" (type &optional table))
@@ -1351,7 +1365,9 @@ Returns a table of all matching tags."
 Optional argument TAGS is a list of tags to search.
 Returns a table of all matching tags."
    (require 'semantic/find)
-   (semantic-find-tags-subclasses-of-type parent (or tags (semanticdb-get-tags table))))
+   (semantic-find-tags-subclasses-of-type
+    parent (or tags (and (slot-boundp table 'tags)
+                         (semanticdb-get-tags table)))))
 
 ;;; Deep Searches
 (cl-defmethod semanticdb-deep-find-tags-by-name-method ((table semanticdb-abstract-table) name &optional tags)
@@ -1360,7 +1376,10 @@ Search in all tags in TABLE, and all components of top level tags in
 TABLE.
 Optional argument TAGS is a list of tags to search.
 Return a table of all matching tags."
-  (semantic-find-tags-by-name name (semantic-flatten-tags-table (or tags (semanticdb-get-tags table)))))
+  (semantic-find-tags-by-name
+   name (semantic-flatten-tags-table
+         (or tags (and (slot-boundp table 'tags)
+                       (semanticdb-get-tags table))))))
 
 (cl-defmethod semanticdb-deep-find-tags-by-name-regexp-method ((table semanticdb-abstract-table) regexp &optional tags)
   "In TABLE, find all occurrences of tags matching REGEXP.
@@ -1368,7 +1387,10 @@ Search in all tags in TABLE, and all components of top level tags in
 TABLE.
 Optional argument TAGS is a list of tags to search.
 Return a table of all matching tags."
-  (semantic-find-tags-by-name-regexp regexp (semantic-flatten-tags-table (or tags (semanticdb-get-tags table)))))
+  (semantic-find-tags-by-name-regexp
+   regexp (semantic-flatten-tags-table
+           (or tags (and (slot-boundp table 'tags)
+                         (semanticdb-get-tags table))))))
 
 (cl-defmethod semanticdb-deep-find-tags-for-completion-method ((table semanticdb-abstract-table) prefix &optional tags)
   "In TABLE, find all occurrences of tags matching PREFIX.
@@ -1376,7 +1398,11 @@ Search in all tags in TABLE, and all components of top level tags in
 TABLE.
 Optional argument TAGS is a list of tags to search.
 Return a table of all matching tags."
-  (semantic-find-tags-for-completion prefix (semantic-flatten-tags-table (or tags (semanticdb-get-tags table)))))
+  (semantic-find-tags-for-completion
+   prefix
+   (semantic-flatten-tags-table
+           (or tags (and (slot-boundp table 'tags)
+                         (semanticdb-get-tags table))))))
 
 (provide 'semantic/db-find)
 
