@@ -1,6 +1,6 @@
 ;;; ffap.el --- find file (or url) at point  -*- lexical-binding: t -*-
 
-;; Copyright (C) 1995-2023 Free Software Foundation, Inc.
+;; Copyright (C) 1995-2024 Free Software Foundation, Inc.
 
 ;; Author: Michelangelo Grigni <mic@mathcs.emory.edu>
 ;; Maintainer: emacs-devel@gnu.org
@@ -152,15 +152,15 @@ schemes (e.g. \"ftp\"); in that case, only convert those URLs."
   :group 'ffap
   :version "24.3")
 
-(defcustom ffap-lax-url t
+(defcustom ffap-lax-url nil
   "If non-nil, allow lax URL matching.
 The default non-nil value might produce false URLs in C++ code
 with symbols like \"std::find\".  On the other hand, setting
 this to nil will disable recognition of URLs that are not
-well-formed, such as \"user@host\" or \"<user@host>\"."
+well-formed."
   :type 'boolean
   :group 'ffap
-  :version "25.2")                      ; nil -> t
+  :version "30.1")
 
 (defcustom ffap-ftp-default-user "anonymous"
   "User name in FTP file names generated by `ffap-host-to-filename'.
@@ -554,15 +554,11 @@ Looks at `ffap-ftp-default-user', returns \"\" for \"localhost\"."
     (concat "gopher://" mach "/"))
    ;; www.ncsa.uiuc.edu
    ((and (string-match "\\`w\\(ww\\|eb\\)[-.]" mach))
-    (concat "http://" mach "/"))
+    (concat "https://" mach "/"))
    ;; More cases?
    (ffap-ftp-regexp (ffap-host-to-filename mach))
    ))
 
-(defvaralias 'ffap-newsgroup-regexp 'thing-at-point-newsgroup-regexp)
-(defvaralias 'ffap-newsgroup-heads  'thing-at-point-newsgroup-heads)
-(defalias 'ffap-newsgroup-p 'thing-at-point-newsgroup-p)
-
 (defun ffap-url-p (string)
   "If STRING looks like an URL, return it (maybe improved), else nil."
   (when (and (stringp string) ffap-url-regexp)
@@ -613,6 +609,13 @@ Looks at `ffap-ftp-default-user', returns \"\" for \"localhost\"."
    ((and ffap-url-unwrap-remote (ffap-url-unwrap-remote url)))
    (url)))
 
+(defun ffap-fixup-email (email)
+  "Clean up EMAIL and return it as a mailto: URL."
+  (when (stringp email)
+    (if (string-prefix-p "mailto:" email)
+        email
+      (concat "mailto:" email))))
+
 
 ;;; File Name Handling:
 
@@ -734,6 +737,7 @@ This uses `ffap-file-exists-string', which may try adding suffixes from
 (defvar ffap-alist
   '(
     ("" . ffap-completable)		; completion, slow on some systems
+    ("" . ffap-in-project)		; maybe in the root of the project
     ("\\.info\\'" . ffap-info)		; gzip.info
     ("\\`info/" . ffap-info-2)		; info/emacs
     ("\\`[-[:lower:]]+\\'" . ffap-info-3) ; (emacs)Top [only in the parentheses]
@@ -747,8 +751,10 @@ This uses `ffap-file-exists-string', which may try adding suffixes from
     ("\\.\\([chCH]\\|cc\\|hh\\)\\'" . ffap-c-mode) ; stdio.h
     (fortran-mode . ffap-fortran-mode)
     ("\\.[fF]\\'" . ffap-fortran-mode)
-    (tex-mode . ffap-tex-mode)		; search ffap-tex-path
+    (plain-tex-mode . ffap-tex-mode)    ; search ffap-tex-path
     (latex-mode . ffap-latex-mode)	; similar
+    (plain-TeX-mode . ffap-tex-mode)    ; AUCTeX v14 counterpart
+    (LaTeX-mode . ffap-latex-mode)      ; ditto
     ("\\.\\(tex\\|sty\\|doc\\|cls\\)\\'" . ffap-tex)
     ("\\.bib\\'" . ffap-bib)		; search ffap-bib-path
     ("\\`\\." . ffap-home)		; .emacs, .bashrc, .profile
@@ -797,6 +803,11 @@ to extract substrings.")
 	 (cmp (file-name-completion (file-name-nondirectory name) dir)))
     (and cmp (concat dir cmp))))
 
+(declare-function project-root "project" (project))
+(defun ffap-in-project (name)
+  (when-let (project (project-current))
+    (file-name-concat (project-root project) name)))
+
 (defun ffap-home (name) (ffap-locate-file name t '("~")))
 
 (defun ffap-info (name)
@@ -1055,7 +1066,13 @@ If a given RFC isn't in these then `ffap-rfc-path' is offered."
     (math-mode ",-:$+<>@-Z_[:lower:]~`" "<" "@>;.,!?`:")
     ;; (La)TeX: don't allow braces
     (latex-mode "--:\\\\$+<>@-Z_[:alpha:]~*?" "<@" "@>;.,!:")
-    (tex-mode "--:\\\\$+<>@-Z_[:alpha:]~*?" "<@" "@>;.,!:")
+    (plain-tex-mode "--:\\\\$+<>@-Z_[:alpha:]~*?" "<@" "@>;.,!:")
+    ;; AUCTeX v14 counterparts:
+    (LaTeX-mode "--:\\\\$+<>@-Z_[:alpha:]~*?" "<@" "@>;.,!:")
+    (plain-TeX-mode "--:\\\\$+<>@-Z_[:alpha:]~*?" "<@" "@>;.,!:")
+    ;; XML: don't allow angle brackets
+    (xml-mode "--:\\\\${}+@-Z_[:alpha:]~*?#" "{<@" "@>;.,!:}")
+    (nxml-mode "--:\\\\${}+@-Z_[:alpha:]~*?#" "{<@" "@>;.,!:}")
     )
   "Alist of (MODE CHARS BEG END), where MODE is a symbol.
 This is possibly a major-mode name, or one of the symbols
@@ -1089,12 +1106,12 @@ Suppose the cursor is somewhere that might be near end of file,
 the guessing would position point before punctuation (like comma)
 after the file extension:
 
-  C:\temp\file.log, which contain ....
+  C:\\temp\\file.log, which contain ....
   =============================== (before)
   ---------------- (after)
 
 
-  C:\temp\file.log on Windows or /tmp/file.log on Unix
+  C:\\temp\\file.log on Windows or /tmp/file.log on Unix
   =============================== (before)
   ---------------- (after)
 
@@ -1569,6 +1586,7 @@ Uses the face `ffap' if it is defined, or else `highlight'."
 	   (ffap-fixup-url (or (ffap-url-at-point)
 			       (ffap-gopher-at-point))))
       (ffap-file-at-point)		; may yield url!
+      (ffap-fixup-email (thing-at-point 'email))
       (ffap-fixup-machine (ffap-machine-at-point))))
 
 (defun ffap-prompter (&optional guess suffix)
@@ -1935,12 +1953,12 @@ Only intended for interactive use."
 ;; bindings you would prefer.
 
 (defun ffap-ro-mode-hook ()
-  "Bind `ffap-next' and `ffap-menu' to M-l and M-m, resp."
+  "Bind `ffap-next' and `ffap-menu' to \\`M-l' and \\`M-m', resp."
   (local-set-key "\M-l" 'ffap-next)
   (local-set-key "\M-m" 'ffap-menu))
 
 (defun ffap-gnus-hook ()
-  "Bind `ffap-gnus-next' and `ffap-gnus-menu' to M-l and M-m, resp."
+  "Bind `ffap-gnus-next' and `ffap-gnus-menu' to \\`M-l' and \\`M-m', resp."
   ;; message-id's
   (setq-local thing-at-point-default-mail-uri-scheme "news")
   ;; Note "l", "L", "m", "M" are taken:
@@ -2145,6 +2163,10 @@ Of course if you do not like these bindings, just roll your own!")
   (interactive)
   (eval (cons 'progn ffap-bindings)))
 
+(define-obsolete-variable-alias 'ffap-newsgroup-regexp 'thing-at-point-newsgroup-regexp "30.1")
+(define-obsolete-variable-alias 'ffap-newsgroup-heads  'thing-at-point-newsgroup-heads "30.1")
+(define-obsolete-function-alias 'ffap-newsgroup-p #'thing-at-point-newsgroup-p "30.1")
+
 
 (provide 'ffap)
 
