@@ -1,6 +1,6 @@
 ;;; characters.el --- set syntax and category for multibyte characters  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1997, 2000-2022 Free Software Foundation, Inc.
+;; Copyright (C) 1997, 2000-2024 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -141,10 +141,10 @@ with L, LRE, or LRO Unicode bidi character type.")
 ;; Chinese characters (Unicode)
 (modify-category-entry '(#x2E80 . #x312F) ?|)
 (modify-category-entry '(#x3190 . #x33FF) ?|)
-(modify-category-entry '(#x3400 . #x4DB5) ?C)
-(modify-category-entry '(#x4E00 . #x9FD5) ?C)
-(modify-category-entry '(#x3400 . #x9FD5) ?c)
-(modify-category-entry '(#x3400 . #x9FD5) ?|)
+(modify-category-entry '(#x3400 . #x4DBF) ?C)
+(modify-category-entry '(#x4E00 . #x9FFF) ?C)
+(modify-category-entry '(#x3400 . #x9FFF) ?c)
+(modify-category-entry '(#x3400 . #x9FFF) ?|)
 (modify-category-entry '(#xF900 . #xFAFF) ?C)
 (modify-category-entry '(#xF900 . #xFAFF) ?c)
 (modify-category-entry '(#xF900 . #xFAFF) ?|)
@@ -152,6 +152,9 @@ with L, LRE, or LRO Unicode bidi character type.")
 (modify-category-entry '(#x20000 . #x2FFFF) ?|)
 (modify-category-entry '(#x20000 . #x2FFFF) ?C)
 (modify-category-entry '(#x20000 . #x2FFFF) ?c)
+(modify-category-entry '(#x30000 . #x323AF) ?|)
+(modify-category-entry '(#x30000 . #x323AF) ?C)
+(modify-category-entry '(#x30000 . #x323AF) ?c)
 
 
 ;; Chinese character set (GB2312)
@@ -181,7 +184,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 
 (dolist (c '(chinese-cns11643-1 chinese-cns11643-2 chinese-cns11643-3
 	     chinese-cns11643-4 chinese-cns11643-5 chinese-cns11643-6
-	     chinese-cns11643-7))
+	     chinese-cns11643-7 chinese-cns11643-15))
   (map-charset-chars #'modify-category-entry c ?c)
   (if (eq c 'chinese-cns11643-1)
       (map-charset-chars #'modify-category-entry c ?C #x4421 #x7E7E)
@@ -227,7 +230,7 @@ with L, LRE, or LRO Unicode bidi character type.")
 (modify-category-entry #x1B001 ?H)
 (modify-category-entry #x1B11F ?H)
 (modify-category-entry '(#x1B150 . #x1B152) ?H)
-(modify-category-entry '(#x1B002 . #x1B11E) ?H) ; Hentiagana
+(modify-category-entry '(#x1B002 . #x1B11E) ?H) ; Hentaigana
 
 (modify-category-entry '(#x1AFF0 . #x1B1FF) ?j)
 
@@ -303,8 +306,10 @@ with L, LRE, or LRO Unicode bidi character type.")
     (setq charsets (cdr charsets))))
 (modify-category-entry '(#x600 . #x6ff) ?b)
 (modify-category-entry '(#x870 . #x8ff) ?b)
-(modify-category-entry '(#xfb50 . #xfdff) ?b)
+(modify-category-entry '(#xfb50 . #xfdcf) ?b)
+(modify-category-entry '(#xfdf0 . #xfdff) ?b)
 (modify-category-entry '(#xfe70 . #xfefe) ?b)
+(modify-category-entry '(#x10ec0 . #x10eff) ?b)
 
 ;; Cyrillic character set (ISO-8859-5)
 
@@ -614,6 +619,9 @@ with L, LRE, or LRO Unicode bidi character type.")
   ;; Cyrillic Extended-C
   (modify-category-entry '(#x1C80 . #x1C8F) ?y)
 
+  ;; Cyrillic Extended-D
+  (modify-category-entry '(#x1E030 . #x1E08F) ?y)
+
   ;; space characters (see section 6.2 in the Unicode Standard)
   (set-case-syntax ?  " " tbl)
   (setq c #x2000)
@@ -687,6 +695,16 @@ with L, LRE, or LRO Unicode bidi character type.")
     (set-case-syntax c "_" tbl)
     (setq c (1+ c)))
 
+  ;; Traditional Mongolian
+  (setq c #x1800)
+  (while (<= c #x180A)
+    (set-case-syntax c "." tbl)
+    (setq c (1+ c)))
+  (setq c #x11660)
+  (while (<= c #x1166C)
+    (set-case-syntax c "." tbl)
+    (setq c (1+ c)))
+
   ;; Coptic
   ;; There's no Coptic category.  However, Coptic letters that are
   ;; part of the Greek block above get the Greek category, and those
@@ -1282,16 +1300,25 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x2B1B . #x2B1C)
 	   (#x2B50 . #x2B50)
 	   (#x2B55 . #x2B55)
-	   (#x2E80 . #x303E)
-	   (#x3040 . #x3247)
+	   (#x2E80 . #x2E99)
+           (#x2E9B . #x2EF3)
+           (#x2F00 . #x2FD5)
+           (#x2FF0 . #x2FFF)
+           (#x3000 . #x303E)
+	   (#x3041 . #x3096)
+           (#x3099 . #x30FF)
+           (#x3105 . #x312F)
+           (#x3131 . #x31E3)
+           (#x31EF . #x31EF)
+           (#x31F0 . #x3247)
 	   (#x3250 . #x4DBF)
-	   (#x4E00 . #x9FFF)
+	   (#x4E00 . #xA48C)
 	   (#xA490 . #xA4C6)
-	   (#xA960 . #xA97F)
+	   (#xA960 . #xA97C)
 	   (#xAC00 . #xD7A3)
 	   (#xF900 . #xFAFF)
 	   (#xFE10 . #xFE19)
-	   (#xFE30 . #xFE6F)
+	   (#xFE30 . #xFE6B)
 	   (#xFF01 . #xFF60)
 	   (#xFFE0 . #xFFE6)
 	   (#x16FE0 . #x16FE4)
@@ -1299,8 +1326,14 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x17000 . #x187F7)
 	   (#x18800 . #x18AFF)
 	   (#x18B00 . #x18CD5)
-	   (#x1AFF0 . #x1AFFF)
-	   (#x1B000 . #x1B152)
+           (#x18D00 . #x18D08)
+	   (#x1AFF0 . #x1AFF3)
+           (#x1AFF5 . #x1AFFB)
+           (#x1AFFD . #x1AFFE)
+	   (#x1B000 . #x1B122)
+           (#x1B132 . #x1B132)
+           (#x1B150 . #x1B152)
+           (#x1B155 . #x1B155)
 	   (#x1B164 . #x1B167)
 	   (#x1B170 . #x1B2FB)
 	   (#x1F004 . #x1F004)
@@ -1308,7 +1341,12 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1F18E . #x1F18E)
 	   (#x1F191 . #x1F19A)
 	   (#x1F1AD . #x1F1AD)
-	   (#x1F200 . #x1F320)
+	   (#x1F200 . #x1F202)
+           (#x1F210 . #x1F23B)
+           (#x1F240 . #x1F248)
+           (#x1F250 . #x1F251)
+           (#x1F260 . #x1F265)
+           (#x1F300 . #x1F320)
 	   (#x1F32D . #x1F335)
 	   (#x1F337 . #x1F37C)
 	   (#x1F37E . #x1F393)
@@ -1333,10 +1371,11 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1F6CC . #x1F6CC)
 	   (#x1F6D0 . #x1F6D2)
 	   (#x1F6D5 . #x1F6D7)
-	   (#x1F6DD . #x1F6DF)
+	   (#x1F6DC . #x1F6DF)
 	   (#x1F6EB . #x1F6EC)
 	   (#x1F6F4 . #x1F6FC)
-	   (#x1F7E0 . #x1F7F0)
+	   (#x1F7E0 . #x1F7EB)
+           (#x1F7F0 . #x1F7F0)
 	   (#x1F90C . #x1F93A)
 	   (#x1F93C . #x1F945)
 	   (#x1F947 . #x1F9FF)
@@ -1344,19 +1383,186 @@ with L, LRE, or LRO Unicode bidi character type.")
 	   (#x1FA60 . #x1FA6D)
 	   (#x1FA70 . #x1FA74)
 	   (#x1FA78 . #x1FA7C)
-	   (#x1FA80 . #x1FA86)
-	   (#x1FA90 . #x1FAAC)
-	   (#x1FAB0 . #x1FABA)
-	   (#x1FAC0 . #x1FAC5)
-	   (#x1FAD0 . #x1FAD9)
-	   (#x1FAE0 . #x1FAE7)
-	   (#x1FAF0 . #x1FAF6)
+	   (#x1FA80 . #x1FA88)
+	   (#x1FA90 . #x1FABD)
+	   (#x1FABF . #x1FAC5)
+	   (#x1FACE . #x1FADB)
+	   (#x1FAE0 . #x1FAE8)
+	   (#x1FAF0 . #x1FAF8)
 	   (#x1FB00 . #x1FB92)
 	   (#x20000 . #x2FFFF)
 	   (#x30000 . #x3FFFF))))
   (dolist (elt l)
     (set-char-table-range char-width-table elt 2)))
 
+;; A: East Asian "Ambiguous" characters.
+(let ((l '((#x00A1 . #x00A1)
+	   (#x00A4 . #x00A4)
+	   (#x00A7 . #x00A8)
+	   (#x00AA . #x00AA)
+	   (#x00AD . #x00AE)
+	   (#x00B0 . #x00B4)
+	   (#x00B6 . #x00BA)
+	   (#x00BC . #x00BF)
+	   (#x00C6 . #x00C6)
+	   (#x00D0 . #x00D0)
+	   (#x00D7 . #x00D8)
+	   (#x00E0 . #x00E1)
+	   (#x00E6 . #x00E6)
+	   (#x00E8 . #x00EA)
+	   (#x00EC . #x00ED)
+	   (#x00F0 . #x00F0)
+	   (#x00F2 . #x00F3)
+	   (#x00F7 . #x00FA)
+	   (#x00FC . #x00FC)
+	   (#x00FE . #x00FE)
+	   (#x0101 . #x0101)
+	   (#x0111 . #x0111)
+	   (#x0113 . #x0113)
+	   (#x011B . #x011B)
+	   (#x0126 . #x0127)
+	   (#x012B . #x012B)
+	   (#x0131 . #x0133)
+	   (#x0138 . #x0138)
+	   (#x013F . #x0142)
+	   (#x0144 . #x0144)
+	   (#x0148 . #x014B)
+	   (#x014D . #x014D)
+	   (#x0152 . #x0153)
+	   (#x0166 . #x0167)
+	   (#x016B . #x016B)
+	   (#x01CE . #x01CE)
+	   (#x01D0 . #x01D0)
+	   (#x01D2 . #x01D2)
+	   (#x01D4 . #x01D4)
+	   (#x01D6 . #x01D6)
+	   (#x01D8 . #x01D8)
+	   (#x01DA . #x01DA)
+	   (#x01DC . #x01DC)
+	   (#x0251 . #x0251)
+	   (#x0261 . #x0261)
+	   (#x02C4 . #x02C4)
+	   (#x02C7 . #x02C7)
+	   (#x02C9 . #x02CB)
+	   (#x02CD . #x02CD)
+	   (#x02D0 . #x02D0)
+	   (#x02D8 . #x02DB)
+	   (#x02DD . #x02DD)
+	   (#x02DF . #x02DF)
+	   (#x0300 . #x036F)
+	   (#x0391 . #x03A1)
+	   (#x03A3 . #x03A9)
+	   (#x03B1 . #x03C1)
+	   (#x03C3 . #x03C9)
+	   (#x0401 . #x0401)
+	   (#x0410 . #x044F)
+	   (#x0451 . #x0451)
+	   (#x2010 . #x2010)
+	   (#x2013 . #x2016)
+	   (#x2018 . #x2019)
+	   (#x201C . #x201D)
+	   (#x2020 . #x2022)
+	   (#x2024 . #x2027)
+	   (#x2030 . #x2030)
+	   (#x2032 . #x2033)
+	   (#x2035 . #x2035)
+	   (#x203E . #x203E)
+	   (#x2074 . #x2074)
+	   (#x207F . #x207F)
+	   (#x2081 . #x2084)
+	   (#x20AC . #x20AC)
+	   (#x2103 . #x2103)
+	   (#x2105 . #x2105)
+	   (#x2109 . #x2109)
+	   (#x2113 . #x2113)
+	   (#x2116 . #x2116)
+	   (#x2121 . #x2122)
+	   (#x2126 . #x2126)
+	   (#x212B . #x212B)
+	   (#x2153 . #x2154)
+	   (#x215B . #x215E)
+	   (#x2160 . #x216B)
+	   (#x2170 . #x2179)
+	   (#x2189 . #x2189)
+	   (#x2190 . #x2199)
+	   (#x21B8 . #x21B9)
+	   (#x21D2 . #x21D2)
+	   (#x21D4 . #x21D4)
+	   (#x21E7 . #x21E7)
+	   (#x2200 . #x2200)
+	   (#x2202 . #x2203)
+	   (#x2207 . #x2208)
+	   (#x220B . #x220B)
+	   (#x220F . #x220F)
+	   (#x2211 . #x2211)
+	   (#x2215 . #x2215)
+	   (#x221A . #x221A)
+	   (#x221D . #x2220)
+	   (#x2223 . #x2223)
+	   (#x2225 . #x2225)
+	   (#x2227 . #x222C)
+	   (#x222E . #x222E)
+	   (#x2234 . #x2237)
+	   (#x223C . #x223D)
+	   (#x2248 . #x2248)
+	   (#x224C . #x224C)
+	   (#x2252 . #x2252)
+	   (#x2260 . #x2261)
+	   (#x2264 . #x2267)
+	   (#x226A . #x226B)
+	   (#x226E . #x226F)
+	   (#x2282 . #x2283)
+	   (#x2286 . #x2287)
+	   (#x2295 . #x2295)
+	   (#x2299 . #x2299)
+	   (#x22A5 . #x22A5)
+	   (#x22BF . #x22BF)
+	   (#x2312 . #x2312)
+	   (#x2460 . #x24E9)
+	   (#x24EB . #x254B)
+	   (#x2550 . #x2573)
+	   (#x2580 . #x258F)
+	   (#x2592 . #x2595)
+	   (#x25A0 . #x25A1)
+	   (#x25A3 . #x25A9)
+	   (#x25B2 . #x25B3)
+	   (#x25B6 . #x25B7)
+	   (#x25BC . #x25BD)
+	   (#x25C0 . #x25C1)
+	   (#x25C6 . #x25C8)
+	   (#x25CE . #x25D1)
+	   (#x25E2 . #x25E5)
+	   (#x25EF . #x25EF)
+	   (#x2605 . #x2606)
+	   (#x260E . #x260F)
+	   (#x261C . #x261C)
+	   (#x261E . #x261E)
+	   (#x2640 . #x2640)
+	   (#x2642 . #x2642)
+	   (#x2660 . #x2661)
+	   (#x2663 . #x2665)
+	   (#x2667 . #x266A)
+	   (#x266C . #x266D)
+	   (#x266F . #x266F)
+	   (#x269E . #x269F)
+	   (#x26BF . #x26BF)
+	   (#x26C6 . #x26CD)
+	   (#x26CF . #x26D3)
+	   (#x26D5 . #x26E1)
+	   (#x26E3 . #x26E3)
+	   (#x26E8 . #x26E9)
+	   (#x26EB . #x26F1)
+	   (#x26F4 . #x26F4)
+	   (#x26F6 . #x26F9)
+	   (#x26FB . #x26FC)
+	   (#x26FE . #x26FF)
+	   (#x273D . #x273D)
+	   (#x2776 . #x277F)
+	   (#x2B56 . #x2B59)
+	   (#x3248 . #x324F))))
+  (dolist (elt l)
+    (set-char-table-range ambiguous-width-chars elt t)))
+
 ;; Other double width
 ;;(map-charset-chars
 ;; (lambda (range ignore) (set-char-table-range char-width-table range 2))
@@ -1390,6 +1596,45 @@ with L, LRE, or LRO Unicode bidi character type.")
 	       (chinese-cns11643-1 (#x2121 . #x427E)))
     (ko_KR nil (korean-ksc5601 (#x2121 . #x2C7E)))))
 
+(defun update-cjk-ambiguous-char-widths (locale-name)
+  "Update character widths for LOCALE-NAME using `ambiguous-width-chars'.
+LOCALE-NAME is the symbol of a CJK locale, such as \\='zh_CN."
+  (let ((slot (assq locale-name cjk-char-width-table-list)))
+    (or slot (error "Unknown locale for CJK language environment: %s"
+		    locale-name))
+    ;; Force recomputation of child table in 'use-cjk-char-width-table'.
+    (setcar (cdr slot) nil)
+    (use-cjk-char-width-table locale-name)))
+
+
+(defcustom cjk-ambiguous-chars-are-wide t
+  "Whether the \"ambiguous-width\" characters take 2 columns on display.
+
+Some of the characters are defined by Unicode as being of \"ambiguous\"
+width: the actual width, either 1 column or 2 columns, should be
+determined at display time, depending on the language context.
+If this variable is non-nil, Emacs will consider these characters as
+full-width, i.e. taking 2 columns; otherwise they are narrow characters
+taking 1 column on display.  Which value is correct depends on the
+fonts being used.  In some CJK locales the fonts are set so that
+these characters are displayed as full-width.  This setting is most
+important for text-mode frames, because there Emacs cannot access the
+metrics of the fonts used by the console or the terminal emulator.
+
+Do not set this directly via `setq'; instead, use `setopt' or the
+Customize commands.  Alternatively, call `update-cjk-ambiguous-char-widths'
+passing it the symbol of the current locale environment, after changing
+the value of the variable with `setq'."
+  :type 'boolean
+  :set (lambda (symbol value)
+         (set-default symbol value)
+         (let ((locsym (get-language-info current-language-environment
+                                          'cjk-locale-symbol)))
+           (when locsym
+             (update-cjk-ambiguous-char-widths locsym))))
+  :version "30.1"
+  :group 'display)
+
 ;; Internal use only.
 ;; Setup char-width-table appropriate for a language environment
 ;; corresponding to LOCALE-NAME (symbol).
@@ -1411,7 +1656,15 @@ with L, LRE, or LRO Unicode bidi character type.")
 				 (car code-range) (cdr code-range)))))
 	(optimize-char-table table)
 	(set-char-table-parent table char-width-table)
-	(setcar (cdr slot) table)))
+        (let ((tbl (make-char-table nil)))
+          (map-char-table
+           (lambda (range _val)
+             (set-char-table-range tbl range
+                                   (if cjk-ambiguous-chars-are-wide 2 1)))
+           ambiguous-width-chars)
+          (optimize-char-table tbl)
+          (set-char-table-parent tbl table)
+	  (setcar (cdr slot) tbl))))
     (setq char-width-table (nth 1 slot))))
 
 (defun use-default-char-width-table ()
@@ -1440,6 +1693,13 @@ Setup `char-width-table' appropriate for non-CJK language environment."
    (set-char-table-range char-script-table range 'tibetan))
  'tibetan)
 
+;; Fix some exceptions that blocks.awk/Blocks.txt couldn't get right.
+(set-char-table-range char-script-table '(#x2ea . #x2eb) 'bopomofo)
+(set-char-table-range char-script-table #xab65 'greek)
+(set-char-table-range char-script-table #x16fe0 'tangut)
+(set-char-table-range char-script-table #x16fe1 'nushu)
+
+
 
 ;;; Setting unicode-category-table.
 
@@ -1493,6 +1753,9 @@ Setup `char-width-table' appropriate for non-CJK language environment."
 (aset char-acronym-table #x202D "LRO")	   ; LEFT-TO-RIGHT OVERRIDE
 (aset char-acronym-table #x202E "RLO")	   ; RIGHT-TO-LEFT OVERRIDE
 (aset char-acronym-table #x2060 "WJ")	   ; WORD JOINER
+(aset char-acronym-table #x2066 "LRI")	   ; LEFT-TO-RIGHT ISOLATE
+(aset char-acronym-table #x2067 "RLI")	   ; RIGHT-TO-LEFT ISOLATE
+(aset char-acronym-table #x2069 "PDI")	   ; POP DIRECTIONAL ISOLATE
 (aset char-acronym-table #x206A "ISS")	   ; INHIBIT SYMMETRIC SWAPPING
 (aset char-acronym-table #x206B "ASS")	   ; ACTIVATE SYMMETRIC SWAPPING
 (aset char-acronym-table #x206C "IAFS")    ; INHIBIT ARABIC FORM SHAPING
@@ -1517,18 +1780,53 @@ Setup `char-width-table' appropriate for non-CJK language environment."
   (aset char-acronym-table (+ #xE0021 i) (format " %c TAG" (+ 33 i))))
 (aset char-acronym-table #xE007F "->|TAG") ; CANCEL TAG
 
+(dotimes (i 256)
+  (let* ((vs-number (1+ i))
+         (codepoint (if (< i 16)
+                        (+ #xfe00 i)
+                      (+ #xe0100 i -16)))
+         (delimiter (cond ((<= vs-number 9) "0")
+                          ((<= vs-number 99) "")
+                          (t " "))))
+    (aset char-acronym-table codepoint
+          (format "VS%s%s" delimiter vs-number))))
+
+;; We can't use the \N{name} things here, because this file is used
+;; too early in the build process.
+(defvar bidi-control-characters
+  '(#x200e                           ; ?\N{left-to-right mark}
+    #x200f                           ; ?\N{right-to-left mark}
+    #x061c                           ; ?\N{arabic letter mark}
+    #x202a			     ; ?\N{left-to-right embedding}
+    #x202b			     ; ?\N{right-to-left embedding}
+    #x202d			     ; ?\N{left-to-right override}
+    #x202e			     ; ?\N{right-to-left override}
+    #x2066			     ; ?\N{left-to-right isolate}
+    #x2067			     ; ?\N{right-to-left isolate}
+    #x2068			     ; ?\N{first strong isolate}
+    #x202c			     ; ?\N{pop directional formatting}
+    #x2069)                          ; ?\N{pop directional isolate}
+  "List of bidirectional control characters.")
+
+(defun bidi-string-strip-control-characters (string)
+  "Strip bidi control characters from STRING and return the result."
+  (apply #'string (seq-filter (lambda (char)
+                                (not (memq char bidi-control-characters)))
+                              string)))
+
 (defun update-glyphless-char-display (&optional variable value)
   "Make the setting of `glyphless-char-display-control' take effect.
 This function updates the char-table `glyphless-char-display',
 and is intended to be used in the `:set' attribute of the
 option `glyphless-char-display'."
-  (when value
+  (when variable
     (set-default variable value))
   (dolist (elt value)
     (let ((target (car elt))
 	  (method (cdr elt)))
-      (or (memq method '(zero-width thin-space empty-box acronym hex-code))
-	  (error "Invalid glyphless character display method: %s" method))
+      (unless (memq method '( zero-width thin-space empty-box
+                              acronym hex-code bidi-control))
+	(error "Invalid glyphless character display method: %s" method))
       (cond ((eq target 'c0-control)
 	     (glyphless-set-char-table-range glyphless-char-display
 					     #x00 #x1F method)
@@ -1542,25 +1840,31 @@ option `glyphless-char-display'."
 					     #x80 #x9F method))
 	    ((eq target 'variation-selectors)
 	     (glyphless-set-char-table-range glyphless-char-display
-					     #xFE00 #xFE0F method))
-	    ((eq target 'format-control)
+					     #xFE00 #xFE0F method)
+             (glyphless-set-char-table-range glyphless-char-display
+					     #xE0100 #xE01EF method))
+	    ((or (eq target 'format-control)
+                 (eq target 'bidi-control))
 	     (when unicode-category-table
 	       (map-char-table
                 (lambda (char category)
-                  (if (eq category 'Cf)
-                      (let ((this-method method)
-                            from to)
-                        (if (consp char)
-                            (setq from (car char) to (cdr char))
-                          (setq from char to char))
-                        (while (<= from to)
-                          (when (/= from #xAD)
-                            (if (eq method 'acronym)
-                                (setq this-method
-                                      (aref char-acronym-table from)))
+                  (when (eq category 'Cf)
+                    (let ((this-method method)
+                          from to)
+                      (if (consp char)
+                          (setq from (car char) to (cdr char))
+                        (setq from char to char))
+                      (while (<= from to)
+                        (when (/= from #xAD)
+                          (when (eq method 'acronym)
+                            (setq this-method
+                                  (or (aref char-acronym-table from)
+                                      "UNK")))
+                          (when (or (eq target 'format-control)
+                                    (memq from bidi-control-characters))
                             (set-char-table-range glyphless-char-display
-                                                  from this-method))
-                          (setq from (1+ from))))))
+                                                  from this-method)))
+                        (setq from (1+ from))))))
 		unicode-category-table)))
 	    ((eq target 'no-font)
 	     (set-char-table-extra-slot glyphless-char-display 0 method))
@@ -1576,6 +1880,19 @@ option `glyphless-char-display'."
     (set-char-table-range chartable (cons from to) method)))
 
 ;;; Control of displaying glyphless characters.
+(define-widget 'glyphless-char-display-method 'lazy
+  "Display method for glyphless characters."
+  :group 'mule
+  :format "%v"
+  :value 'thin-space
+  :type
+  '(choice
+    (const :tag "Don't display" zero-width)
+    (const :tag "Display as thin space" thin-space)
+    (const :tag "Display as empty box" empty-box)
+    (const :tag "Display acronym" acronym)
+    (const :tag "Display hex code in a box" hex-code)))
+
 (defcustom glyphless-char-display-control
   '((format-control . thin-space)
     (variation-selectors . thin-space)
@@ -1594,13 +1911,19 @@ GROUP must be one of these symbols:
                     such as U+200C (ZWNJ), U+200E (LRM), but
                     excluding characters that have graphic images,
                     such as U+00AD (SHY).
-  `variation-selectors': U+FE00..U+FE0F, used for choosing between
-                         glyph variations (e.g. Emoji vs Text
-                         presentation).
-  `no-font':        characters for which no suitable font is found.
-                    For character terminals, characters that cannot
-                    be encoded by `terminal-coding-system' or those
-                    for which the terminal has no glyphs.
+  `bidi-control':   A subset of `format-control', but only characters
+                    that are relevant for bidirectional formatting control,
+                    like U+2069 (PDI) and U+202B (RLE).
+  `variation-selectors':
+                    Characters in the range U+FE00..U+FE0F and
+                    U+E0100..U+E01EF, used for choosing between
+                    glyph variations, such as Emoji vs Text
+                    presentation, of the preceding character(s).
+  `no-font':        For GUI frames, characters for which no
+                    suitable font is found; for text-mode frames,
+                    characters that cannot be encoded by
+                    `terminal-coding-system' or those for which
+                    the terminal has no glyphs.
 
 METHOD must be one of these symbols:
   `zero-width': don't display.
@@ -1621,36 +1944,12 @@ visual representation of these characters."
   :version "28.1"
   :type '(alist :key-type (symbol :tag "Character Group")
 		:value-type (symbol :tag "Display Method"))
-  :options '((c0-control
-	      (choice (const :tag "Don't display" zero-width)
-		      (const :tag "Display as thin space" thin-space)
-		      (const :tag "Display as empty box" empty-box)
-		      (const :tag "Display acronym" acronym)
-		      (const :tag "Display hex code in a box" hex-code)))
-	     (c1-control
-	      (choice (const :tag "Don't display" zero-width)
-		      (const :tag "Display as thin space" thin-space)
-		      (const :tag "Display as empty box" empty-box)
-		      (const :tag "Display acronym" acronym)
-		      (const :tag "Display hex code in a box" hex-code)))
-	     (format-control
-	      (choice (const :tag "Don't display" zero-width)
-		      (const :tag "Display as thin space" thin-space)
-		      (const :tag "Display as empty box" empty-box)
-		      (const :tag "Display acronym" acronym)
-		      (const :tag "Display hex code in a box" hex-code)))
-	     (variation-selectors
-	      (choice (const :tag "Don't display" zero-width)
-		      (const :tag "Display as thin space" thin-space)
-		      (const :tag "Display as empty box" empty-box)
-		      (const :tag "Display acronym" acronym)
-		      (const :tag "Display hex code in a box" hex-code)))
-	     (no-font
-	      (choice (const :tag "Don't display" zero-width)
-		      (const :tag "Display as thin space" thin-space)
-		      (const :tag "Display as empty box" empty-box)
-		      (const :tag "Display acronym" acronym)
-		      (const :tag "Display hex code in a box" hex-code))))
+  :options '((c0-control glyphless-char-display-method)
+	     (c1-control glyphless-char-display-method)
+	     (format-control glyphless-char-display-method)
+	     (bidi-control glyphless-char-display-method)
+	     (variation-selectors glyphless-char-display-method)
+	     (no-font (glyphless-char-display-method :value hex-code)))
   :set 'update-glyphless-char-display
   :group 'display)
 
