@@ -1,6 +1,6 @@
-;;; latin-ltx.el --- Quail package for TeX-style input -*-coding: utf-8;-*-
+;;; latin-ltx.el --- Quail package for TeX-style input -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2001-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2001-2024 Free Software Foundation, Inc.
 ;; Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
 ;;   2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -8,7 +8,7 @@
 
 ;; Author: TAKAHASHI Naoto <ntakahas@m17n.org>
 ;;         Dave Love <fx@gnu.org>
-;; Keywords: multilingual, input, Greek, i18n
+;; Keywords: multilingual, input method, i18n
 
 ;; This file is part of GNU Emacs.
 
@@ -67,7 +67,7 @@ system, including many technical ones.  Examples:
     (and (characterp char) (< char 128)))
 
   (defmacro latin-ltx--define-rules (&rest rules)
-    (load "uni-name")
+    (load "uni-name" nil t)
     (let ((newrules ()))
       (dolist (rule rules)
         (pcase rule
@@ -105,10 +105,11 @@ system, including many technical ones.  Examples:
                   (setq rules (delq c rules)))
                 (message "Conflict for %S: %S"
                          (car rule) (apply #'string conflicts)))))))
-      (let ((inputs (mapcar #'car newrules)))
-        (setq inputs (delete-dups inputs))
-        (message "latin-ltx: %d rules (+ %d conflicts)!"
-                 (length inputs) (- (length newrules) (length inputs))))
+      (let* ((inputs (delete-dups (mapcar #'car newrules)))
+             (conflicts (- (length newrules) (length inputs))))
+        (unless (zerop conflicts)
+          (message "latin-ltx: %d rules (+ %d conflicts)!"
+                   (length inputs) conflicts)))
       `(quail-define-rules ,@(nreverse newrules)))))
 
 (latin-ltx--define-rules
@@ -241,12 +242,15 @@ system, including many technical ones.  Examples:
  ((lambda (name char)
     ;; "GREEK SMALL LETTER PHI" (which is \phi) and "GREEK PHI SYMBOL"
     ;; (which is \varphi) are reversed in `ucs-names', so we define
-    ;; them manually.
-    (unless (string-match-p "\\<PHI\\>" name)
+    ;; them manually.  Also ignore "GREEK SMALL LETTER EPSILON" and
+    ;; add the correct value for \epsilon manually.
+    (unless (string-match-p "\\<GREEK SMALL LETTER \\(?:EPSILON\\|PHI\\)\\>"
+                            name)
       (concat "\\" (funcall (if (match-end 1) #' capitalize #'downcase)
                             (match-string 2 name)))))
   "\\`GREEK \\(?:SMALL\\|CAPITA\\(L\\)\\) LETTER \\([^- ]+\\)\\'")
 
+ ("\\epsilon" ?ϵ)
  ("\\phi" ?ϕ)
  ("\\Box" ?□)
  ("\\Bumpeq" ?≎)
@@ -276,13 +280,17 @@ system, including many technical ones.  Examples:
  ("\\Vdash" ?⊩)
  ("\\Vert" ?‖)
  ("\\Vvdash" ?⊪)
+ ("\\above" ?┴)
  ("\\aleph" ?ℵ)
  ("\\amalg" ?∐)
  ("\\angle" ?∠)
+ ("\\aoint" ?∳)
  ("\\approx" ?≈)
  ("\\approxeq" ?≊)
+ ("\\asmash" ?⬆)
  ("\\ast" ?∗)
  ("\\asymp" ?≍)
+ ("\\atop" ?¦)
  ("\\backcong" ?≌)
  ("\\backepsilon" ?∍)
  ("\\backprime" ?‵)
@@ -291,11 +299,18 @@ system, including many technical ones.  Examples:
  ("\\backslash" ?\\)
  ("\\barwedge" ?⊼)
  ("\\because" ?∵)
+ ("\\begin" ?\〖)
+ ("\\below" ?┬)
  ("\\beth" ?ℶ)
  ("\\between" ?≬)
  ("\\bigcap" ?⋂)
  ("\\bigcirc" ?◯)
  ("\\bigcup" ?⋃)
+ ("\\bigodot" ?⨀)
+ ("\\bigoplus" ?⨁)
+ ("\\bigotimes" ?⨂)
+ ("\\bigsqcup" ?⨆)
+ ("\\biguplus" ?⨄)
  ("\\bigstar" ?★)
  ("\\bigtriangledown" ?▽)
  ("\\bigtriangleup" ?△)
@@ -312,6 +327,7 @@ system, including many technical ones.  Examples:
  ("\\boxminus" ?⊟)
  ("\\boxplus" ?⊞)
  ("\\boxtimes" ?⊠)
+ ("\\bra" ?\⟨)
  ("\\bullet" ?•)
  ("\\bumpeq" ?≏)
  ("\\cap" ?∩)
@@ -328,7 +344,9 @@ system, including many technical ones.  Examples:
  ("\\circledast" ?⊛)
  ("\\circledcirc" ?⊚)
  ("\\circleddash" ?⊝)
+ ("\\close" ?┤)
  ("\\clubsuit" ?♣)
+ ("\\coint" ?∲)
  ("\\coloneq" ?≔)
  ("\\complement" ?∁)
  ("\\cong" ?≅)
@@ -346,8 +364,12 @@ system, including many technical ones.  Examples:
  ("\\dagger" ?†)
  ("\\daleth" ?ℸ)
  ("\\dashv" ?⊣)
+ ("\\Dd" ?ⅅ)
+ ("\\dd" ?ⅆ)
  ("\\ddag" ?‡)
  ("\\ddagger" ?‡)
+ ("\\ddddot" ?⃜)
+ ("\\dddot" ?⃛)
  ("\\ddots" ?⋱)
  ("\\diamond" ?⋄)
  ("\\diamondsuit" ?♢)
@@ -360,8 +382,12 @@ system, including many technical ones.  Examples:
  ("\\downdownarrows" ?⇊)
  ("\\downleftharpoon" ?⇃)
  ("\\downrightharpoon" ?⇂)
+ ("\\dsmash" ?⬇)
+ ("\\ee" ?ⅇ)
  ("\\ell" ?ℓ)
  ("\\emptyset" ?∅)
+ ("\\end" ?\〗)
+ ("\\eqarray" ?█)
  ("\\eqcirc" ?≖)
  ("\\eqcolon" ?≕)
  ("\\eqslantgtr" ?⋝)
@@ -411,16 +437,25 @@ system, including many technical ones.  Examples:
  ("\\heartsuit" ?♥)
  ("\\hookleftarrow" ?↩)
  ("\\hookrightarrow" ?↪)
+ ("\\hphantom" ?⬄)
+ ("\\hsmash" ?⬌)
  ("\\iff" ?⇔)
+ ("\\ii" ?ⅈ)
+ ("\\iiiint" ?⨌)
+ ("\\iiint" ?∭)
+ ("\\iint" ?∬)
  ("\\imath" ?ı)
  ("\\in" ?∈)
  ("\\infty" ?∞)
  ("\\int" ?∫)
  ("\\intercal" ?⊺)
+ ("\\jj" ?ⅉ)
+ ("\\jmath" ?ȷ)
  ("\\langle" ?⟨) ;; Was ?〈, see bug#12948.
  ("\\lbrace" ?{)
  ("\\lbrack" ?\[)
  ("\\lceil" ?⌈)
+ ("\\ldiv" ?∕)
  ("\\ldots" ?…)
  ("\\le" ?≤)
  ("\\leadsto" ?↝)
@@ -449,14 +484,15 @@ system, including many technical ones.  Examples:
  ("\\rhd" ?▷)
  ("\\ll" ?≪)
  ("\\llcorner" ?⌞)
+ ("\\lll" ?⋘)
  ("\\lnapprox" ?⋦)
  ("\\lneq" ?≨)
  ("\\lneqq" ?≨)
  ("\\lnsim" ?⋦)
- ("\\longleftarrow" ?←)
- ("\\longleftrightarrow" ?↔)
- ("\\longmapsto" ?↦)
- ("\\longrightarrow" ?→)
+ ("\\longleftarrow" ?⟵)
+ ("\\longleftrightarrow" ?⟷)
+ ("\\longmapsto" ?⟼)
+ ("\\longrightarrow" ?⟶)
  ("\\looparrowleft" ?↫)
  ("\\looparrowright" ?↬)
  ("\\lozenge" ?✧)
@@ -526,16 +562,25 @@ system, including many technical ones.  Examples:
  ("\\nvdash" ?⊬)
  ("\\nwarrow" ?↖)
  ("\\odot" ?⊙)
+ ("\\oiiint" ?∰)
+ ("\\oiint" ?∯)
  ("\\oint" ?∮)
  ("\\ominus" ?⊖)
  ("\\oplus" ?⊕)
  ("\\oslash" ?⊘)
  ("\\otimes" ?⊗)
+ ("\\overbrace" ?⏞)
+ ("\\overparen" ?⏜)
  ("\\par" ? )
  ("\\parallel" ?∥)
  ("\\partial" ?∂)
  ("\\perp" ?⊥)
+ ("\\phantom" ?⟡)
  ("\\pitchfork" ?⋔)
+ ("\\pppprime" ?⁗)
+ ("\\ppprime" ?‴)
+ ("\\pprime" ?″)
+ ("\\prcue" ?≼)
  ("\\prec" ?≺)
  ("\\precapprox" ?≾)
  ("\\preceq" ?≼)
@@ -545,12 +590,16 @@ system, including many technical ones.  Examples:
  ("\\prime" ?′)
  ("\\prod" ?∏)
  ("\\propto" ?∝)
+ ("\\qdrt" ?∜)
  ("\\qed" ?∎)
  ("\\quad" ? )
  ("\\rangle" ?\⟩) ;; Was ?〉, see bug#12948.
+ ("\\ratio" ?∶)
  ("\\rbrace" ?})
  ("\\rbrack" ?\])
  ("\\rceil" ?⌉)
+ ("\\rddots" ?⋰)
+ ("\\rect" ?▭)
  ("\\rfloor" ?⌋)
  ("\\rightarrow" ?→)
  ("\\rightarrowtail" ?↣)
@@ -562,6 +611,8 @@ system, including many technical ones.  Examples:
  ("\\rightrightarrows" ?⇉)
  ("\\rightthreetimes" ?⋌)
  ("\\risingdotseq" ?≓)
+ ("\\rrect" ?▢)
+ ("\\sdiv" ?⁄)
  ("\\rtimes" ?⋊)
  ("\\sbs" ?﹨)
  ("\\searrow" ?↘)
@@ -574,6 +625,7 @@ system, including many technical ones.  Examples:
  ("\\smallamalg" ?∐)
  ("\\smallsetminus" ?∖)
  ("\\smallsmile" ?⌣)
+ ("\\smash" ?⬍)
  ("\\smile" ?⌣)
  ("\\spadesuit" ?♠)
  ("\\sphericalangle" ?∢)
@@ -624,12 +676,16 @@ system, including many technical ones.  Examples:
  ("\\ulcorner" ?⌜)
  ("\\uparrow" ?↑)
  ("\\updownarrow" ?↕)
+ ("\\underbar" ?▁)
+ ("\\underbrace" ?⏟)
+ ("\\underparen" ?⏝)
  ("\\upleftharpoon" ?↿)
  ("\\uplus" ?⊎)
  ("\\uprightharpoon" ?↾)
  ("\\upuparrows" ?⇈)
  ("\\urcorner" ?⌝)
  ("\\u{i}" ?ĭ)
+ ("\\vbar" ?│)
  ("\\vDash" ?⊨)
 
  ((lambda (name char)
@@ -640,6 +696,7 @@ system, including many technical ones.  Examples:
       (concat "\\var" (downcase (match-string 1 name)))))
   "\\`GREEK \\([^- ]+\\) SYMBOL\\'")
 
+ ("\\varepsilon" ?ε)
  ("\\varphi" ?φ)
  ("\\varprime" ?′)
  ("\\varpropto" ?∝)
@@ -651,14 +708,39 @@ system, including many technical ones.  Examples:
  ("\\vee" ?∨)
  ("\\veebar" ?⊻)
  ("\\vert" ?|)
+ ("\\vphantom" ?⇳)
  ("\\wedge" ?∧)
  ("\\wp" ?℘)
  ("\\wr" ?≀)
 
- ("\\Bbb{N}" ?ℕ)			; AMS commands for blackboard bold
- ("\\Bbb{P}" ?ℙ)			; Also sometimes \mathbb.
+ ("\\Bbb{A}" ?𝔸)			; AMS commands for blackboard bold
+ ("\\Bbb{B}" ?𝔹)			; Also sometimes \mathbb.
+ ("\\Bbb{C}" ?ℂ)
+ ("\\Bbb{D}" ?𝔻)
+ ("\\Bbb{E}" ?𝔼)
+ ("\\Bbb{F}" ?𝔽)
+ ("\\Bbb{G}" ?𝔾)
+ ("\\Bbb{H}" ?ℍ)
+ ("\\Bbb{I}" ?𝕀)
+ ("\\Bbb{J}" ?𝕁)
+ ("\\Bbb{K}" ?𝕂)
+ ("\\Bbb{L}" ?𝕃)
+ ("\\Bbb{M}" ?𝕄)
+ ("\\Bbb{N}" ?ℕ)
+ ("\\Bbb{O}" ?𝕆)
+ ("\\Bbb{P}" ?ℙ)
+ ("\\Bbb{Q}" ?ℚ)
  ("\\Bbb{R}" ?ℝ)
+ ("\\Bbb{S}" ?𝕊)
+ ("\\Bbb{T}" ?𝕋)
+ ("\\Bbb{U}" ?𝕌)
+ ("\\Bbb{V}" ?𝕍)
+ ("\\Bbb{W}" ?𝕎)
+ ("\\Bbb{X}" ?𝕏)
+ ("\\Bbb{Y}" ?𝕐)
  ("\\Bbb{Z}" ?ℤ)
+ ("\\Bbb{1}" ?𝟙)
+ ("\\Bbb{2}" ?𝟚)
  ("--" ?–)
  ("---" ?—)
  ;; We used to use ~ for NBSP but that's inconvenient and may even look like
@@ -702,7 +784,9 @@ system, including many technical ones.  Examples:
  ("\\ldq" ?\“)
  ("\\rdq" ?\”)
  ("\\defs" ?≙)				; per fuzz/zed
- ;; ("\\sqrt[3]" ?∛)
+ ("\\sqrt" ?√)
+ ("\\sqrt[3]" ?∛)
+ ("\\sqrt[4]" ?∜)
  ("\\llbracket" ?\〚) 			; stmaryrd
  ("\\rrbracket" ?\〛)
  ;; ("\\lbag" ?\〚) 			; fuzz
