@@ -1,6 +1,6 @@
 ;;; ediff-mult.el --- support for multi-file/multi-buffer processing in Ediff  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1995-2020 Free Software Foundation, Inc.
+;; Copyright (C) 1995-2024 Free Software Foundation, Inc.
 
 ;; Author: Michael Kifer <kifer@cs.stonybrook.edu>
 ;; Package: ediff
@@ -43,11 +43,11 @@
 ;;                                 (...) ...)
 ;;         The function ediff-make-new-meta-list-element can be used to create
 ;;         2nd and subsequent elements of that list (i.e., after the
-;;         description header). See ediff-make-new-meta-list-element for the
+;;         description header).  See ediff-make-new-meta-list-element for the
 ;;         explanation of the two nil placeholders in such elements.
 ;;
 ;;         There is API for extracting the components of the members of the
-;;         above list. Search for `API for ediff-meta-list' for details.
+;;         above list.  Search for `API for ediff-meta-list' for details.
 ;;
 ;;	   HEADER must be a list of SIX elements (nil or string):
 ;;             (regexp metaobj1 metaobj2 metaobj3 merge-save-buffer
@@ -113,10 +113,9 @@
 (require 'ediff-wind)
 (require 'ediff-util)
 
-
 ;; meta-buffer
-(ediff-defvar-local ediff-meta-buffer nil "")
-(ediff-defvar-local ediff-parent-meta-buffer nil "")
+(ediff-defvar-local ediff-meta-buffer nil)
+(ediff-defvar-local ediff-parent-meta-buffer nil)
 ;; the registry buffer
 (defvar ediff-registry-buffer nil)
 
@@ -129,7 +128,7 @@
 (defconst ediff-meta-buffer-verbose-message "Ediff Session Group Panel: %s
 
 Useful commands (type ? to hide them and free up screen):
-     button2, v, or RET over session record:   start that Ediff session
+     mouse-2, v, or RET over session record:   start that Ediff session
      M:\tin sessions invoked from here, brings back this group panel
      R:\tdisplay the registry of active Ediff sessions
      h:\tmark session for hiding (toggle)
@@ -145,44 +144,41 @@ Useful commands (type ? to hide them and free up screen):
 
 (ediff-defvar-local ediff-meta-buffer-map nil
   "The keymap for the meta buffer.")
-(defvar ediff-dir-diffs-buffer-map
-  (let ((map (make-sparse-keymap)))
-    (suppress-keymap map)
-    (define-key map "q" 'ediff-bury-dir-diffs-buffer)
-    (define-key map " " 'next-line)
-    (define-key map "n" 'next-line)
-    (define-key map "\C-?" 'previous-line)
-    (define-key map "p" 'previous-line)
-    (define-key map "C" 'ediff-dir-diff-copy-file)
-    (define-key map  [mouse-2] 'ediff-dir-diff-copy-file)
-    (define-key map [delete] 'previous-line)
-    (define-key map [backspace] 'previous-line)
-    map)
-  "The keymap to be installed in the buffer showing differences between
-directories.")
+(defvar-keymap ediff-dir-diffs-buffer-map
+  :doc "Keymap for buffer showing differences between directories."
+  :suppress t
+  "q"           #'ediff-bury-dir-diffs-buffer
+  "SPC"         #'next-line
+  "n"           #'next-line
+  "DEL"         #'previous-line
+  "p"           #'previous-line
+  "C"           #'ediff-dir-diff-copy-file
+  "<mouse-2>"   #'ediff-dir-diff-copy-file
+  "<delete>"    #'previous-line
+  "<backspace>" #'previous-line)
 
 ;; Variable specifying the action to take when the use invokes ediff in the
 ;; meta buffer.  This is usually ediff-registry-action or ediff-filegroup-action
-(ediff-defvar-local ediff-meta-action-function nil "")
+(ediff-defvar-local ediff-meta-action-function nil)
 ;; Tells ediff-update-meta-buffer how to redraw it
-(ediff-defvar-local ediff-meta-redraw-function nil "")
+(ediff-defvar-local ediff-meta-redraw-function nil)
 ;; Tells ediff-filegroup-action and similar procedures how to invoke Ediff for
 ;; the sessions in a given session group
-(ediff-defvar-local ediff-session-action-function nil "")
+(ediff-defvar-local ediff-session-action-function nil)
 
-(ediff-defvar-local ediff-metajob-name nil "")
+(ediff-defvar-local ediff-metajob-name nil)
 
 ;; buffer used to collect custom diffs from individual sessions in the group
-(ediff-defvar-local ediff-meta-diff-buffer nil "")
+(ediff-defvar-local ediff-meta-diff-buffer nil)
 
 ;; t means recurse into subdirs when deciding which files have same contents
-(ediff-defvar-local ediff-recurse-to-subdirectories nil "")
+(ediff-defvar-local ediff-recurse-to-subdirectories nil)
 
 ;; history var to use for filtering groups of files
-(defvar ediff-filtering-regexp-history nil "")
+(defvar ediff-filtering-regexp-history nil)
 
 (defcustom ediff-default-filtering-regexp nil
-  "The default regular expression used as a filename filter in multifile comparisons.
+  "Default regular expression used as a filename filter in multifile comparisons.
 Should be a sexp.  For instance (car ediff-filtering-regexp-history) or nil."
   :type 'sexp                           ; yuck - why not just a regexp?
   :risky t)
@@ -196,14 +192,14 @@ Should be a sexp.  For instance (car ediff-filtering-regexp-history) or nil."
 ;; buffer, this means ediff is still working on the pair.
 ;; Eq-status of a file is t if the file equals some other file in the same
 ;; group.
-(ediff-defvar-local ediff-meta-list nil "")
+(ediff-defvar-local ediff-meta-list nil)
 
-(ediff-defvar-local ediff-meta-session-number nil "")
+(ediff-defvar-local ediff-meta-session-number nil)
 
 
 ;; the difference list between directories in a directory session group
-(ediff-defvar-local ediff-dir-difference-list nil "")
-(ediff-defvar-local ediff-dir-diffs-buffer nil "")
+(ediff-defvar-local ediff-dir-difference-list nil)
+(ediff-defvar-local ediff-dir-diffs-buffer nil)
 
 ;; The registry of Ediff sessions.  A list of control buffers.
 (defvar ediff-session-registry nil)
@@ -234,8 +230,8 @@ on `ediff-quit', `ediff-suspend', or `ediff-quit-session-group-hook'."
 (make-obsolete-variable 'ediff-before-session-group-setup-hooks nil "27.1")
 
 (defcustom ediff-after-session-group-setup-hook nil
-  "Hooks run just after a meta-buffer controlling a session group, such as
-ediff-directories, is run."
+  "Hooks run just after a meta-buffer controlling a session group is run.
+One example of this is `ediff-directories'."
   :type 'hook)
 (defcustom ediff-quit-session-group-hook nil
   "Hooks run just before exiting a session group."
@@ -253,8 +249,8 @@ This means that you can set different bindings for different kinds of meta
 buffers."
   :type 'hook)
 
-;; Buffer holding the multi-file patch.  Local to the meta buffer
-(ediff-defvar-local ediff-meta-patchbufer nil "")
+(ediff-defvar-local ediff-meta-patchbufer nil
+  "Buffer holding the multi-file patch.  Local to the meta buffer.")
 
 ;;; API for ediff-meta-list
 
@@ -368,8 +364,8 @@ buffers."
 
 
 (ediff-defvar-local ediff-verbose-help-enabled nil
-  "If t, display redundant help in ediff-directories and other meta buffers.
-Toggled by ediff-toggle-verbose-help-meta-buffer" )
+  "If t, display redundant help in `ediff-directories' and other meta buffers.
+Toggled by `ediff-toggle-verbose-help-meta-buffer'.")
 
 ;; Toggle verbose help in meta-buffers
 ;; TODO: Someone who understands all this can make it better.
@@ -414,12 +410,11 @@ Toggled by ediff-toggle-verbose-help-meta-buffer" )
       '(menu-item "Show Manual" ediff-documentation
 		  :help "Display Ediff's manual"))
 
-    (or (ediff-one-filegroup-metajob)
-	(progn
-	  (define-key ediff-meta-buffer-map "=" nil)
-	  (define-key ediff-meta-buffer-map "==" 'ediff-meta-mark-equal-files)
-	  (define-key ediff-meta-buffer-map "=m" 'ediff-meta-mark-equal-files)
-	  (define-key ediff-meta-buffer-map "=h" 'ediff-meta-mark-equal-files)))
+    (unless (ediff-one-filegroup-metajob)
+      (define-key ediff-meta-buffer-map "=" nil)
+      (define-key ediff-meta-buffer-map "==" #'ediff-meta-mark-equal-files)
+      (define-key ediff-meta-buffer-map "=m" #'ediff-meta-mark-equal-files)
+      (define-key ediff-meta-buffer-map "=h" #'ediff-meta-mark-equal-files))
 
 
     (define-key menu-map [ediff-next-meta-item]
@@ -431,7 +426,7 @@ Toggled by ediff-toggle-verbose-help-meta-buffer" )
 
 
   (if ediff-no-emacs-help-in-control-buffer
-      (define-key ediff-meta-buffer-map  "\C-h"  'ediff-previous-meta-item))
+      (define-key ediff-meta-buffer-map  "\C-h"  #'ediff-previous-meta-item))
   (define-key ediff-meta-buffer-map [mouse-2] ediff-meta-action-function)
 
   (use-local-map ediff-meta-buffer-map)
@@ -462,7 +457,9 @@ Commands:
 
 (defun ediff-next-meta-item (count)
   "Move to the next item in Ediff registry or session group buffer.
-Moves in circular fashion.  With numeric prefix arg, skip this many items."
+Moves in circular fashion.
+
+With numeric prefix arg COUNT, skip this many items."
   (interactive "p")
   (or count (setq count 1))
   (let (overl)
@@ -490,7 +487,9 @@ Moves in circular fashion.  With numeric prefix arg, skip this many items."
 
 (defun ediff-previous-meta-item (count)
   "Move to the previous item in Ediff registry or session group buffer.
-Moves in circular fashion.  With numeric prefix arg, skip this many items."
+Moves in circular fashion.
+
+With numeric prefix arg COUNT, skip this many items."
   (interactive "p")
   (or count (setq count 1))
   (let (overl)
@@ -613,28 +612,28 @@ behavior."
     (if (ediff-nonempty-string-p merge-autostore-dir)
 	(setq merge-autostore-dir
 	      (file-name-as-directory merge-autostore-dir)))
-    (setq common (ediff-intersection lis1 lis2 #'string=))
+    (setq common (seq-intersection lis1 lis2 #'string=))
 
     ;; In merge with ancestor jobs, we don't intersect with lis3.
     ;; If there is no ancestor, we'll offer to merge without the ancestor.
     ;; So, we intersect with lis3 only when we are doing 3-way file comparison
     (if (and lis3 (ediff-comparison-metajob3 jobname))
-	(setq common (ediff-intersection common lis3 #'string=)))
+        (setq common (seq-intersection common lis3 #'string=)))
 
     ;; copying is needed because sort sorts via side effects
-    (setq common (sort (ediff-copy-list common) 'string-lessp))
+    (setq common (sort (copy-sequence common) #'string-lessp))
 
     ;; compute difference list
-    (setq difflist (ediff-set-difference
-		    (ediff-union (ediff-union lis1 lis2 #'string=)
-				 lis3
-				 #'string=)
+    (setq difflist (seq-difference
+                    (seq-union (seq-union lis1 lis2 #'string=)
+                               lis3
+                               #'string=)
 		    common
 		    #'string=)
 	  difflist (delete "."  difflist)
 	  ;; copying is needed because sort sorts via side effects
-	  difflist (sort (ediff-copy-list (delete ".." difflist))
-			 'string-lessp))
+          difflist (sort (copy-sequence (delete ".." difflist))
+			 #'string-lessp))
 
     (setq difflist (mapcar (lambda (elt) (cons elt 1)) difflist))
 
@@ -731,7 +730,7 @@ behavior."
 	  )
 
     ;; copying is needed because sort sorts via side effects
-    (setq common (sort (ediff-copy-list common) 'string-lessp))
+    (setq common (sort (copy-sequence common) #'string-lessp))
 
     ;; return result
     (cons
@@ -838,14 +837,14 @@ behavior."
 		(ediff-draw-dir-diffs ediff-dir-difference-list))
 	    (define-key
 	      ediff-meta-buffer-map "h" 'ediff-mark-for-hiding-at-pos)
-	    (define-key ediff-meta-buffer-map "x" 'ediff-hide-marked-sessions)
+	    (define-key ediff-meta-buffer-map "x" #'ediff-hide-marked-sessions)
 	    (define-key
-	      ediff-meta-buffer-map "m" 'ediff-mark-for-operation-at-pos)
+	      ediff-meta-buffer-map "m" #'ediff-mark-for-operation-at-pos)
 	    (define-key ediff-meta-buffer-map "u" nil)
 	    (define-key
-	      ediff-meta-buffer-map "um" 'ediff-unmark-all-for-operation)
+	      ediff-meta-buffer-map "um" #'ediff-unmark-all-for-operation)
 	    (define-key
-	      ediff-meta-buffer-map "uh" 'ediff-unmark-all-for-hiding)
+	      ediff-meta-buffer-map "uh" #'ediff-unmark-all-for-hiding)
 
 	    (define-key ediff-meta-buffer-map
 	      [menu-bar ediff-meta-mode ediff-hide-marked-sessions]
@@ -878,7 +877,7 @@ behavior."
 		     '(menu-item "Collect diffs" ediff-collect-custom-diffs
 				 :help "Collect custom diffs of marked sessions in buffer `*Ediff Multifile Diffs*'"))
 		   (define-key
-		     ediff-meta-buffer-map "P" 'ediff-collect-custom-diffs))
+		     ediff-meta-buffer-map "P" #'ediff-collect-custom-diffs))
 		  ((ediff-patch-metajob jobname)
 		   (define-key ediff-meta-buffer-map
 		     [menu-bar ediff-meta-mode ediff-meta-show-patch]
@@ -886,8 +885,8 @@ behavior."
 				 :help "Show the multi-file patch associated with this group session"))
 		   (define-key
 		     ediff-meta-buffer-map "P" 'ediff-meta-show-patch)))
-	    (define-key ediff-meta-buffer-map "^" 'ediff-up-meta-hierarchy)
-	    (define-key ediff-meta-buffer-map "D" 'ediff-show-dir-diffs)
+	    (define-key ediff-meta-buffer-map "^" #'ediff-up-meta-hierarchy)
+	    (define-key ediff-meta-buffer-map "D" #'ediff-show-dir-diffs)
 
 	    (define-key ediff-meta-buffer-map
 	      [menu-bar ediff-meta-mode ediff-up-meta-hierarchy]
@@ -1172,7 +1171,7 @@ behavior."
 	  ;; abbreviate the file name, if file exists
 	  (if (and (not (stringp fname)) (< file-size -1))
 	      "-------"		; file doesn't exist
-	    (ediff-truncate-string-left
+	    (string-truncate-left
 	     (ediff-abbreviate-file-name fname)
 	     max-filename-width)))))))
 
@@ -1235,7 +1234,7 @@ behavior."
       (insert "\t\t*** Directory Differences ***\n")
       (insert "
 Useful commands:
-  C,button2: over file name -- copy this file to directory that doesn't have it
+  C,mouse-2: over file name -- copy this file to directory that doesn't have it
           q: hide this buffer
       n,SPC: next line
       p,DEL: previous line\n\n")
@@ -1266,7 +1265,7 @@ Useful commands:
 	(if (= (mod membership-code ediff-membership-code1) 0) ; dir1
 	    (let ((beg (point)))
 	      (insert (format "%-27s"
-			      (ediff-truncate-string-left
+			      (string-truncate-left
 			       (ediff-abbreviate-file-name
 				(if (file-directory-p (concat dir1 file))
 				    (file-name-as-directory file)
@@ -1281,7 +1280,7 @@ Useful commands:
 	(if (= (mod membership-code ediff-membership-code2) 0) ; dir2
 	    (let ((beg (point)))
 	      (insert (format "%-26s"
-			      (ediff-truncate-string-left
+			      (string-truncate-left
 			       (ediff-abbreviate-file-name
 				(if (file-directory-p (concat dir2 file))
 				    (file-name-as-directory file)
@@ -1295,7 +1294,7 @@ Useful commands:
 	    (if (= (mod membership-code ediff-membership-code3) 0) ; dir3
 		(let ((beg (point)))
 		  (insert (format " %-25s"
-				  (ediff-truncate-string-left
+				  (string-truncate-left
 				   (ediff-abbreviate-file-name
 				    (if (file-directory-p (concat dir3 file))
 					(file-name-as-directory file)
@@ -1428,7 +1427,7 @@ Useful commands:
 This is a registry of all active Ediff sessions.
 
 Useful commands:
-     button2, `v', RET over a session record:  switch to that session
+     mouse-2, `v', RET over a session record:  switch to that session
      M over a session record:  display the associated session group
      R in any Ediff session:   display session registry
      n,SPC: next session
@@ -1533,7 +1532,9 @@ Useful commands:
 	(ediff-overlay-put overl 'ediff-meta-session-number session-number))))
 
 (defun ediff-mark-for-hiding-at-pos (unmark)
-  "Mark session for hiding.  With prefix arg, unmark."
+  "Mark session for hiding.
+
+With prefix arg UNMARK, unmark instead."
   (interactive "P")
   (let* ((pos (ediff-event-point last-command-event))
 	 (meta-buf (ediff-event-buffer last-command-event))
@@ -1543,10 +1544,9 @@ Useful commands:
     (ediff-mark-session-for-hiding info unmark)
     (ediff-next-meta-item 1)
     (save-excursion
-      (ediff-update-meta-buffer meta-buf nil session-number))
-    ))
+      (ediff-update-meta-buffer meta-buf nil session-number))))
 
-;; Returns whether session was marked or unmarked
+;; Return whether session was marked or unmarked.
 (defun ediff-mark-session-for-hiding (info unmark)
   (let (ignore)
     (cond ((eq unmark 'mark) (setq unmark nil))
@@ -1562,7 +1562,9 @@ Useful commands:
 
 
 (defun ediff-mark-for-operation-at-pos (unmark)
-  "Mark session for a group operation.  With prefix arg, unmark."
+  "Mark session for a group operation.
+
+With prefix arg UNMARK, unmark instead."
   (interactive "P")
   (let* ((pos (ediff-event-point last-command-event))
 	 (meta-buf (ediff-event-buffer last-command-event))
@@ -1591,7 +1593,9 @@ Useful commands:
 
 
 (defun ediff-hide-marked-sessions (unhide)
-  "Hide marked sessions.  With prefix arg, unhide."
+  "Hide marked sessions.
+
+With prefix arg UNHIDE, unhide instead."
   (interactive "P")
   (let ((grp-buf (ediff-get-group-buffer ediff-meta-list))
 	(meta-list (cdr ediff-meta-list))
@@ -1671,7 +1675,7 @@ Useful commands:
 		(setq custom-diff-buf ediff-custom-diff-buffer)))))
 
     (or (ediff-buffer-live-p meta-diff-buff)
-	(user-error "Ediff: something wrong--killed multiple diff's buffer"))
+        (user-error "Ediff: Something wrong--killed multiple diff's buffer"))
 
     (cond ((ediff-buffer-live-p custom-diff-buf)
 	   ;; for live session buffers we do them first because the user may
@@ -1808,11 +1812,9 @@ all marked sessions must be active."
 		 (ediff-show-meta-buffer session-buf)
 	       (setq regexp
 		     (read-string
-		      (if (stringp default-regexp)
-			  (format
-			   "Filter filenames through regular expression (default %s): "
-			   default-regexp)
-			"Filter filenames through regular expression: ")
+                      (format-prompt
+                       "Filter filenames through regular expression"
+                       default-regexp)
 		      nil
 		      'ediff-filtering-regexp-history
 		      (eval ediff-default-filtering-regexp t)))
@@ -1857,7 +1859,6 @@ all marked sessions must be active."
 	    ;; handle an individual session with a live control buffer
 	    ((ediff-buffer-live-p session-buf)
 	     (ediff-with-current-buffer session-buf
-	       (setq ediff-mouse-pixel-position (mouse-pixel-position))
 	       (ediff-recenter 'no-rehighlight)))
 
 	    ((ediff-problematic-session-p info)
@@ -2001,7 +2002,6 @@ all marked sessions must be active."
 	    (ediff-show-meta-buffer ctl-buf t)
 	  ;; it's a session buffer -- invoke go back to session
 	  (ediff-with-current-buffer ctl-buf
-	    (setq ediff-mouse-pixel-position (mouse-pixel-position))
 	    (ediff-recenter 'no-rehighlight)))
       (beep)
       (message "You've selected a stale session --- try again")
@@ -2052,14 +2052,14 @@ all marked sessions must be active."
 	      ((and
 		(setq wind
 		      (ediff-get-visible-buffer-window ediff-registry-buffer))
-		(ediff-window-display-p))
+                (display-graphic-p))
 	       (select-window wind)
 	       (other-window 1)
 	       (set-window-buffer (selected-window) meta-buf))
 	      (t (ediff-skip-unsuitable-frames 'ok-unsplittable)
 		 (set-window-buffer (selected-window) meta-buf)))
 	))
-    (if (and (ediff-window-display-p)
+    (if (and (display-graphic-p)
 	     (window-live-p
 	      (setq wind (ediff-get-visible-buffer-window meta-buf))))
 	(progn
@@ -2113,14 +2113,14 @@ all marked sessions must be active."
 		(select-window ediff-window-B))
 	       ((and (setq wind
 			   (ediff-get-visible-buffer-window ediff-meta-buffer))
-		     (ediff-window-display-p))
+                     (display-graphic-p))
 		(select-window wind)
 		(other-window 1)
 		(set-window-buffer (selected-window) ediff-registry-buffer))
 	       (t (ediff-skip-unsuitable-frames 'ok-unsplittable)
 		  (set-window-buffer (selected-window) ediff-registry-buffer)))
 	))
-    (if (ediff-window-display-p)
+    (if (display-graphic-p)
 	(progn
 	  (setq frame
 		(window-frame
@@ -2131,7 +2131,7 @@ all marked sessions must be active."
     ))
 
 ;;;###autoload
-(defalias 'eregistry 'ediff-show-registry)
+(defalias 'eregistry #'ediff-show-registry)
 
 ;; If meta-buf doesn't exist, it is created.  In that case, id doesn't have a
 ;; parent meta-buf
@@ -2320,7 +2320,7 @@ If this is a session registry buffer then just bury it."
 	 (meta-patchbuf ediff-meta-patchbufer)
 	 session-buf beg-marker end-marker)
 
-    (if (or (file-directory-p file) (string-match "/dev/null" file))
+    (if (or (file-directory-p file) (string-match null-device file))
 	(user-error "`%s' is not an ordinary file" (file-name-as-directory file)))
     (setq session-buf (ediff-get-session-buffer info)
 	  beg-marker (ediff-get-session-objB-name info)
