@@ -111,18 +111,6 @@ along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.  */
 # endif
 #endif
 
-/* This isn't perfect, as some systems might have the page file in
-   another place.  Also, I suspect that the time stamp of that file
-   might also change when Windows enlarges the file due to
-   insufficient VM.  Still, this seems to be the most reliable way;
-   the alternative (of using GetSystemTimes) won't work on laptops
-   that hibernate, because the system clock is stopped then.  Other
-   possibility would be to run "net statistics workstation" and parse
-   the output, but that's gross.  So this should do; if the file is
-   not there, the boot time will be returned as zero, and filelock.c
-   already handles that.  */
-#define BOOT_TIME_FILE "C:/pagefile.sys"
-
 /* ============================================================ */
 
 /* Here, add any special hacks needed to make Emacs work on this
@@ -220,7 +208,7 @@ extern struct tm * sys_localtime (const time_t *);
 
 /* Unlike MS and mingw.org, MinGW64 doesn't define gai_strerror as an
    inline function in a system header file, and instead seems to
-   require to link against ws2_32.a.  But we don't want to link with
+   require linking against ws2_32.a.  But we don't want to link with
    -lws2_32, as that would make Emacs dependent on the respective DLL.
    So MinGW64 is amply punished here by the following:  */
 #undef HAVE_GAI_STRERROR
@@ -615,7 +603,7 @@ typedef unsigned int EMACS_UINT;
 
    Starting with MSVC 5.0, we must also place the uninitialized data
    into its own section.  VC5 intermingles uninitialized data from the CRT
-   between Emacs' static uninitialized data and its public uninitialized
+   between Emacs's static uninitialized data and its public uninitialized
    data.  A separate .bss section for Emacs groups both static and
    public uninitialized together.
 
