@@ -225,7 +225,7 @@ tzlookup (Lisp_Object zone, bool settz)
 
   if (NILP (zone))
     return local_tz;
-  else if (BASE_EQ (zone, make_fixnum (0)) || BASE2_EQ (zone, Qt))
+  else if (BASE_EQ (zone, make_fixnum (0)) || EQ (zone, Qt))
     {
       zone_string = "UTC0";
       new_tz = utc_tz;
@@ -234,7 +234,7 @@ tzlookup (Lisp_Object zone, bool settz)
     {
       bool plain_integer = FIXNUMP (zone);
 
-      if (BASE2_EQ (zone, Qwall))
+      if (EQ (zone, Qwall))
 	zone_string = 0;
       else if (STRINGP (zone))
 	zone_string = SSDATA (ENCODE_SYSTEM (zone));
@@ -414,9 +414,11 @@ decode_float_time (double t, struct lisp_time *result)
   else
     {
       int scale = double_integer_scale (t);
-      /* FIXME: `double_integer_scale` often returns values that are
-         "pessimistic" (i.e. larger than necessary), so 3.5 gets converted
-         to (7881299347898368 . 2251799813685248) rather than (7 . 2).
+      /* Because SCALE treats trailing zeros in T as significant,
+	 on typical platforms with IEEE floating point
+	 (time-convert 3.5 t) yields (7881299347898368 . 2251799813685248),
+	 a precision of 2**-51 s, not (7 . 2), a precision of 0.5 s.
+	 Although numerically correct, this generates largish integers.
          On 64bit systems, this should not matter very much, tho.  */
       eassume (scale < flt_radix_power_size);
 
@@ -514,8 +516,8 @@ timespec_ticks (struct timespec t)
   /* For speed, use intmax_t arithmetic if it will do.  */
   intmax_t accum;
   if (FASTER_TIMEFNS
-      && !INT_MULTIPLY_WRAPV (t.tv_sec, TIMESPEC_HZ, &accum)
-      && !INT_ADD_WRAPV (t.tv_nsec, accum, &accum))
+      && !ckd_mul (&accum, t.tv_sec, TIMESPEC_HZ)
+      && !ckd_add (&accum, accum, t.tv_nsec))
     return make_int (accum);
 
   /* Fall back on bignum arithmetic.  */
@@ -543,7 +545,7 @@ lisp_time_hz_ticks (struct lisp_time t, Lisp_Object hz)
       /* For speed, use intmax_t arithmetic if it will do.  */
       intmax_t ticks;
       if (FASTER_TIMEFNS && FIXNUMP (t.ticks) && FIXNUMP (t.hz)
-	  && !INT_MULTIPLY_WRAPV (XFIXNUM (t.ticks), XFIXNUM (hz), &ticks))
+	  && !ckd_mul (&ticks, XFIXNUM (t.ticks), XFIXNUM (hz)))
 	return make_int (ticks / XFIXNUM (t.hz)
 			 - (ticks % XFIXNUM (t.hz) < 0));
     }
@@ -1548,7 +1550,7 @@ usage: (decode-time &optional TIME ZONE FORM)  */)
 
   /* Compute SEC from LOCAL_TM.tm_sec and HZ.  */
   Lisp_Object hz = lt.hz, sec;
-  if (BASE_EQ (hz, make_fixnum (1)) || !BASE2_EQ (form, Qt))
+  if (BASE_EQ (hz, make_fixnum (1)) || !EQ (form, Qt))
     sec = make_fixnum (local_tm.tm_sec);
   else
     {
@@ -1557,12 +1559,10 @@ usage: (decode-time &optional TIME ZONE FORM)  */)
       Lisp_Object ticks;
       intmax_t n;
       if (FASTER_TIMEFNS && FIXNUMP (lt.ticks) && FIXNUMP (hz)
-	  && !INT_MULTIPLY_WRAPV (XFIXNUM (hz), local_tm.tm_sec, &n)
-	  && ! (INT_ADD_WRAPV
-		(n, (XFIXNUM (lt.ticks) % XFIXNUM (hz)
-		     + (XFIXNUM (lt.ticks) % XFIXNUM (hz) < 0
-			? XFIXNUM (hz) : 0)),
-		 &n)))
+	  && !ckd_mul (&n, XFIXNUM (hz), local_tm.tm_sec)
+	  && !ckd_add (&n, n, (XFIXNUM (lt.ticks) % XFIXNUM (hz)
+			       + (XFIXNUM (lt.ticks) % XFIXNUM (hz) < 0
+				  ? XFIXNUM (hz) : 0))))
 	ticks = make_int (n);
       else
 	{
@@ -1603,7 +1603,7 @@ check_tm_member (Lisp_Object obj, int offset)
       CHECK_FIXNUM (obj);
       EMACS_INT n = XFIXNUM (obj);
       int i;
-      if (INT_SUBTRACT_WRAPV (n, offset, &i))
+      if (ckd_sub (&i, n, offset))
 	time_overflow ();
       return i;
     }
@@ -1767,10 +1767,8 @@ but new code should not rely on it.  */)
      well, since we accept it as input?  */
   struct lisp_time t;
   enum timeform input_form = decode_lisp_time (time, false, &t, 0);
-  if (NILP (form))
-    form = current_time_list ? Qlist : Qt;
-  if (symbols_with_pos_enabled && SYMBOL_WITH_POS_P (form))
-    form = SYMBOL_WITH_POS_SYM (form);
+  form = (!NILP (form) ? maybe_remove_pos_from_symbol (form)
+	  : current_time_list ? Qlist : Qt);
   if (BASE_EQ (form, Qlist))
     return ticks_hz_list4 (t.ticks, t.hz);
   if (BASE_EQ (form, Qinteger))
