@@ -1,6 +1,6 @@
 ;;; cl-print-tests.el --- Test suite for the cl-print facility.  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2017-2021 Free Software Foundation, Inc.
+;; Copyright (C) 2017-2024 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -25,6 +25,7 @@
 ;;; Code:
 
 (require 'ert)
+(require 'cl-print)
 
 (cl-defstruct (cl-print-tests-struct
                (:constructor cl-print-tests-con))
@@ -59,18 +60,20 @@
 
 (ert-deftest cl-print-tests-ellipsis-string ()
   "Ellipsis expansion works in strings."
-  (let ((print-length 4)
-        (print-level 3))
+  (let ((cl-print-string-length 4))
     (cl-print-tests-check-ellipsis-expansion
      "abcdefg" "\"abcd...\"" "efg")
     (cl-print-tests-check-ellipsis-expansion
      "abcdefghijk" "\"abcd...\"" "efgh...")
-    (cl-print-tests-check-ellipsis-expansion
-     '(1 (2 (3 #("abcde" 0 5 (test t)))))
-     "(1 (2 (3 ...)))" "#(\"abcd...\" 0 5 (test t))")
-    (cl-print-tests-check-ellipsis-expansion
-     #("abcd" 0 1 (bold t) 1 2 (invisible t) 3 4 (italic t))
-     "#(\"abcd\" 0 1 (bold t) ...)" "1 2 (invisible t) ...")))
+    (let ((print-length 4)
+          (print-level 3))
+      (cl-print-tests-check-ellipsis-expansion
+       '(1 (2 (3 #("abcde" 0 5 (test t)))))
+       "(1 (2 (3 ...)))" "#(\"abcd...\" 0 5 (test t))"))
+    (let ((print-length 4))
+      (cl-print-tests-check-ellipsis-expansion
+       #("abcd" 0 1 (bold t) 1 2 (invisible t) 3 4 (italic t))
+       "#(\"abcd\" 0 1 (bold t) ...)" "1 2 (invisible t) ..."))))
 
 (ert-deftest cl-print-tests-ellipsis-struct ()
   "Ellipsis expansion works in structures."
@@ -90,7 +93,7 @@
 (ert-deftest cl-print-tests-ellipsis-circular ()
   "Ellipsis expansion works with circular objects."
   (let ((wide-obj (list 0 1 2 3 4))
-        (deep-obj `(0 (1 (2 (3 (4))))))
+        (deep-obj (list 0 (list 1 (list 2 (list 3 (list 4))))))
         (print-length 4)
         (print-level 3))
     (setf (nth 4 wide-obj) wide-obj)
@@ -113,7 +116,7 @@
     (should pos)
     (setq value (get-text-property pos 'cl-print-ellipsis result))
     (should (equal expected result))
-    (should (equal expanded (with-output-to-string (cl-print-expand-ellipsis
+    (should (equal expanded (with-output-to-string (cl-print--expand-ellipsis
                                                     value nil))))))
 
 (defun cl-print-tests-check-ellipsis-expansion-rx (obj expected expanded)
@@ -122,7 +125,7 @@
          (value (get-text-property pos 'cl-print-ellipsis result)))
     (should (string-match expected result))
     (should (string-match expanded (with-output-to-string
-                                     (cl-print-expand-ellipsis value nil))))))
+                                     (cl-print--expand-ellipsis value nil))))))
 
 (ert-deftest cl-print-tests-print-to-string-with-limit ()
   (let* ((thing10 (make-list 10 'a))
