@@ -174,7 +174,8 @@ SEVERITY-PREDICATE is used to setup
     (flymake-tests--with-flymake
         ("some-problems.h")
       (flymake-goto-next-error)
-      (should (eq 'flymake-warning (face-at-point)))
+      ;; implicit-int was promoted from warning to error in GCC 14
+      (should (memq (face-at-point) '(flymake-warning flymake-error)))
       (flymake-goto-next-error)
       (should (eq 'flymake-error (face-at-point)))
       (should-error (flymake-goto-next-error nil nil t)))
@@ -213,6 +214,7 @@ SEVERITY-PREDICATE is used to setup
 
 (ert-deftest dummy-backends ()
   "Test many different kinds of backends."
+  (let ((debug-on-error nil))
   (with-temp-buffer
     (cl-letf
         (((symbol-function 'error-backend)
@@ -291,7 +293,7 @@ SEVERITY-PREDICATE is used to setup
         (should (eq 'flymake-warning (face-at-point))) ; dolor
         (flymake-goto-next-error)
         (should (eq 'flymake-error (face-at-point))) ; prognata
-        (should-error (flymake-goto-next-error nil nil t))))))
+        (should-error (flymake-goto-next-error nil nil t)))))))
 
 (ert-deftest recurrent-backend ()
   "Test a backend that calls REPORT-FN multiple times."
