@@ -23,6 +23,7 @@
 ;;; Code:
 
 (require 'ert)
+(require 'cl-lib)
 
 (defun keymap-tests--make-keymap-test (fun)
   (should (eq (car (funcall fun)) 'keymap))
@@ -356,6 +357,13 @@ g .. h		foo
                  "C-x <right>"))
   (should (equal (key-description [M-H-right] [?\C-x])
                  "C-x M-H-<right>"))
+  ;; Treat latin-1 correctly vs meta.  (Bug#59305)
+  (should (equal (key-description "olá")
+                 "o l á"))
+  (should (equal (key-description (string ?o ?l ?á))
+                 "o l á"))
+  (should (equal (key-description (unibyte-string ?o ?l ?á))
+                 "o l M-a"))
   (should (equal (single-key-description 'home)
                  "<home>"))
   (should (equal (single-key-description 'home t)
@@ -470,10 +478,18 @@ g .. h		foo
        ert-keymap-duplicate
        "a" #'next-line
        "a" #'previous-line))
-  (should-error
-   (define-keymap
-       "a" #'next-line
-       "a" #'previous-line)))
+  (let ((msg ""))
+    ;; FIXME: It would be nicer to use `current-message' rather than override
+    ;; `message', but `current-message' returns always nil in batch mode :-(
+    (cl-letf (((symbol-function 'message)
+               (lambda (fmt &rest args) (setq msg (apply #'format fmt args)))))
+      (should
+       (string-match "duplicate"
+                     (progn
+                       (define-keymap
+                         "a" #'next-line
+                         "a" #'previous-line)
+                       msg))))))
 
 (ert-deftest keymap-unset-test-remove-and-inheritance ()
   "Check various behaviors of keymap-unset.  (Bug#62207)"
